# -*- coding: utf-8 -*-
"""
Created on Thu Mar 30 15:31:22 2023

@author: kruis
"""

#%% Import libraries, functions and set paths
import os
import sys
import pandas as pd
import json
import numpy as np
from datetime import datetime
import py_cwt2d
import math

def findAverageBedProfile(hs_dict, file_info):
    tot_bed_profile = np.zeros_like(hs_dict.get(file_info.loc[0, 'Date Time']))
    count = np.zeros_like(tot_bed_profile)
    for key, value in hs_dict.items():
        nan_mask = np.isnan(value)
        count = count + np.where(nan_mask, 0, 1)
        value[np.isnan(value)] = 0
        tot_bed_profile = tot_bed_profile + value       
    avg_bed_profile = tot_bed_profile / count
    avg_bed_profile[np.isnan(avg_bed_profile)] = 0
    return avg_bed_profile    

# paths
strco_path = r'C:/Users/kruis/Documents/Thesis/Python/StreamCoords'
file_info_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES/file_info.pkl'
hotspot_path = r'C:\Users\kruis\Documents\Thesis\Python\hotspot_data.xlsx'
out_path = r'C:\Users\kruis\Documents\Thesis\Python\waveletOut'

#%% Initiate scrypt

# Settings
hotspot = 'E'
wavelet_name = 'mexh'   # works for 'mexh' and 'morlet'

# Check if chosen wavelet is valid 
if wavelet_name not in ['morlet', 'mexh']:
    print('Choose either the morlet or mexh wavelet')
    sys.exit()

# Rework settings to be workable 
hs_string = 'ABCDEFGHIJ'
index = hs_string.find(hotspot)

# Get the hotspot specific information
hotspot_data = pd.read_excel(hotspot_path)

# get transformed sn file
sn_name = 'HS_' + hotspot + '_sn.pkl'
sn = pd.read_pickle(os.path.join(strco_path,sn_name))

# Define minimum and maximum values of the Stream Coords grid
ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())

# Open dictionary for reading
strco_dict_name = 'HS_' + hotspot + '_max.json'                                                     
with open(os.path.join(strco_path, strco_dict_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
hs_dict = {}
for key, value in serialized_dict.items():
    hs_dict[datetime.fromisoformat(key)] = np.array(value)

# Load file info data about this hotspot that has inputs linking to the dictionary
file_info = pd.read_pickle(file_info_path)
file_info = file_info[file_info['Hotspot Letter'] == hotspot]
file_info = file_info[file_info.isin(hs_dict.keys()).any(axis=1)]
file_info.reset_index(inplace=True, drop=True)

# Determine average bed profile during all measurements
avg_bed_profile = findAverageBedProfile(hs_dict, file_info)
np.save(os.path.join(r'C:/Users/kruis/Documents/Thesis/Python/waveletOut', hotspot,'average_bed.npy'), avg_bed_profile)

# Set some parameters for the analysis that remain the same for all time indices
numrows, numcols = avg_bed_profile.shape
margin = 100
diff = numcols + 2*margin - numrows
vertical_padding = np.zeros((int(diff/2),numcols+2*margin))


#%% Torrence and Compo scale selection
lowerbound = 1              # lowerbound fourier wavelength of the dunes
upperbound = 2*ds_max       # upperbound foerier wavelength of the dunes

omega0 = 2 # see motherwavelets toolbox
m = 2 # order of gaus for mexh (see motherwavelets toolbox)
lambda_morlet_divided_s = 4*math.pi/(omega0+math.sqrt(2+omega0**2))
lambda_mexh_divided_s = 2*math.pi/(math.sqrt(m+0.5))

s0 = 2
dj = 0.125      # the lower, the more different wavelengths can be identified
dx = 1
N = avg_bed_profile.shape[1]

J = 1/dj*math.log2(N*dx/s0)
sJ = s0*2**(J*dj)
J=math.ceil(J) # amount of intervals between scales, minimum = for morlet, according to T&C

# Calculate lower and upper scale, depending on the wavelet used
if wavelet_name == 'mexh':
    high_freq = lowerbound/lambda_mexh_divided_s
    low_freq = upperbound/lambda_mexh_divided_s
elif wavelet_name == 'morlet':
    high_freq = lowerbound/lambda_morlet_divided_s
    low_freq = upperbound/lambda_morlet_divided_s

# set up a range of scales in logarithmic spacing between 1 and 256 (image width / 2 seems to work ehre)
ss = np.logspace(np.log2(high_freq),np.log2(low_freq),J,base=2)
wavelengths = ss*lambda_mexh_divided_s
# Save array of wavelengths to .npy file
wavelengths_name = 'HS_' + hotspot + '_wavelengths.npy'
np.save(os.path.join(out_path,hotspot,wavelengths_name), wavelengths)

#%% TIME CONSUMING!!! Perform Wavelet Transform
# HS_F takes about 3 hours

for time_index in range(len(file_info)):
    
    print("Current time =", time_index)
    # Get image
    time = file_info.loc[time_index, 'Date Time']
    Z = hs_dict.get(time)
    # remove average bed profile
    Z_avg = Z - avg_bed_profile
    # pad the image with zeros and make it square
    image = np.pad(Z_avg, ((0,0), (margin, margin)), mode='constant')
    image = np.concatenate((vertical_padding, image, vertical_padding), axis=0)
    if image.shape[1] - image.shape[0] == 1:
        image = image[:, :-1]
    
    # Perform wavelet transform
    # calculate the complex pycwt and the wavelet normalizations
    coeffs, wav_norm = py_cwt2d.cwt_2d(image, ss, wavelet_name)
    
    level = len(ss)
    C = 1.0 / (ss[:level] * wav_norm[:level])
    rec3d = C * np.real(coeffs)
    rec3d = rec3d[int(diff/2):int(diff/2+numrows), int(margin):int(margin+numcols), :]
    
    # inverse the reconstructed signal if the morlet wavelet is used
    if wavelet_name == 'morlet':
        rec3d = rec3d * -1
    
    rec3d_name = str(time_index) + '.npy'
    np.save(os.path.join(out_path,hotspot,rec3d_name), rec3d)
   





















