# -*- coding: utf-8 -*-
"""
Created on Sun Jun 18 20:50:18 2023

@author: kruis
"""

#%% INITIATE: 
    
hotspot = 'H'

# wdsmin, wdsmax, wxsmin, wxsmax = 150, 2200, 60, 210          # HS E
# wdsmin, wdsmax, wxsmin, wxsmax = 300, 2200, 50, 200         # HS F
wdsmin, wdsmax, wxsmin, wxsmax = 800, 2800, 0, 280          # HS - H
# wdsmin, wdsmax, wxsmin, wxsmax = 500, 1400, 65, 190         # HS J

dataframe_name = 'HS_' + hotspot + '_all_data.pkl'
# dataframe_name = 'HS_' + hotspot + '_all_data_rerun.pkl'
# dataframe_name = 'HS_' + hotspot + '_ws_small_all_data.pkl'    
    
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import math
from datetime import datetime, timedelta, date
from scipy.optimize import curve_fit

import warnings
warnings.filterwarnings("ignore")

def interpolate_nan_values(arr):
    # Find the indices of the non-NaN values
    non_nan_indices = np.where(~np.isnan(arr))[0]

    # Interpolate the NaN values between the first and last non-NaN values
    interpolated_arr = np.full_like(arr, np.nan)
    interpolated_arr[non_nan_indices[0]:non_nan_indices[-1] + 1] = np.interp(
        np.arange(non_nan_indices[0], non_nan_indices[-1] + 1),
        non_nan_indices,
        arr[non_nan_indices]
    )

    return interpolated_arr

def get_previous_5days(date):
    # Create an empty array to store the result
    previous_days = []

    # Iterate over the range of 5 days
    for i in range(1, 6):
        # Subtract i days from the original date
        previous_day = date - timedelta(days=i)

        # Set the hours and minutes to 0
        previous_day = previous_day.replace(hour=0, minute=0)

        # Add the modified date to the result array
        previous_days.append(previous_day)

    return previous_days

out_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut'
StrCo_path = r'C:/Users/kruis/Documents/Thesis/Python/StreamCoords'

df_all = pd.read_pickle(os.path.join(out_path,dataframe_name))
df_all = df_all.loc[df_all['celerity XY'] < 100]
df_all['dredged perc'] = df_all['dredged volume'] / (df_all['dune volume'] + df_all['dredged volume'])
# Calculate the average 'celerity XY' per time index
average_celerity = df_all.groupby('time index')['celerity XY'].mean()
# Create a new column 'average_celerity' in the DataFrame
df_all['average celerity'] = df_all['time index'].map(average_celerity)

# file_info_name = 'HS_'+hotspot+'_FI_test.pkl'
file_info_name = 'HS_'+hotspot+'_FI.pkl'
file_info = pd.read_pickle(os.path.join(out_path,file_info_name))  
file_info = file_info.reset_index(drop=True)
file_info['time index'] = file_info.index

# get transformed sn file
sn_name = 'HS_' + hotspot + '_sn.pkl'
sn = pd.read_pickle(os.path.join(StrCo_path,sn_name))

# Define minimum and maximum values of the Stream Coords grid
ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())

# import discharge data
Q = pd.read_csv(r'C:/Users/kruis/Documents/Thesis/files_RWS/afvoeren/Pan.csv', delimiter=';')
Q = Q[['datum', 'tijd', 'waarde']]
Q['waarde'] = Q['waarde'].str.replace(',', '.').astype(float)
Q['datum'] = Q['datum'] + ' ' + Q['tijd']
Q['datum'] = pd.to_datetime(Q['datum'], format='%d-%m-%Y %H:%M')
Q = Q.loc[Q['waarde']<10000]
Q = Q.loc[Q['waarde']>0]

# Group the DataFrame by date and calculate the average 'waarde' per day
Q_daily = Q.groupby(Q['datum'].dt.date)['waarde'].mean().reset_index()

# Rename the columns
Q_daily.columns = ['date', 'average_waarde']

# Convert the 'date' column back to datetime format
Q_daily['date'] = pd.to_datetime(Q_daily['date'])

discharge5 = np.zeros(len(file_info))
discharge = np.zeros(len(file_info))
for i, row in file_info.iterrows():
    time = row['Date Time']
    past5days = get_previous_5days(time)
    dfQ5 = Q_daily.loc[Q_daily['date'].isin(past5days)]
    dfQ = Q_daily.loc[Q_daily['date'].dt.date == file_info.loc[i, 'Date Time'].date()]
    discharge[i] = dfQ['average_waarde'].values[0]
    discharge5[i] = dfQ5['average_waarde'].mean() 
file_info['Q'] = discharge
file_info['Q5'] = discharge5

# Remove values outside the specified range
df_filtered = df_all[(df_all['ds'] >= wdsmin) & (df_all['ds'] <= wdsmax) & (df_all['xs'] >= wxsmin) & (df_all['xs'] <= wxsmax)]

extent = [wdsmax - wdsmin, 0, wxsmin+xs_min, wxsmax+xs_min]

# Remove TP values and measurements to close to each other
file_info_filtered = file_info[file_info['TP Value'] == 0]
time_steps = file_info_filtered['time index'].unique()
dff = df_all.loc[df_all['time index'].isin(file_info_filtered['time index'])]

# determine average growth for non dredged areas
overall = dff.loc[(df_all['dredged perc'] == 0) & (df_all['ploughed_max'] ==0)]
overall_mean_h = overall.groupby('time index')['height'].median()
overall_mean_dh = np.diff(np.array(overall_mean_h))
overall_mean_T = overall.groupby('time index')['time'].median()
overall_mean_T = (overall_mean_T - overall_mean_T.min()).dt.total_seconds() / (24 * 60 * 60)
overall_mean_dT = np.diff(np.array(overall_mean_T))
overall_mean_growth = overall_mean_dh / overall_mean_dT
overall_mean_growth_rel = overall_mean_dh / overall_mean_h[:-1] / overall_mean_dT
overall_mean_growth_rel = pd.DataFrame(overall_mean_growth_rel, columns=['height'])
overall_mean_growth_rel = overall_mean_growth_rel.rename(columns={'height': 'growth'})
overall_mean_growth_rel['growth'] = overall_mean_growth

file_info = file_info.merge(overall_mean_growth_rel, how='left', left_on='time index', right_index=True)


#%% PREPROCESS: tracked height no dredging multiple

def createTrackedAll(moment, interval):
    trackname = str(moment) + '.pkl'
    tracked = pd.read_pickle(os.path.join(out_path,hotspot,'Ploeg_new',trackname))
    tracked = tracked.drop('Z', axis=1)
    tracked = tracked.drop('migration angle', axis=1)
    median = tracked.loc[(tracked['time index'] == moment)]['height'].median()
    median = 0.2
    filtered_ids = tracked.loc[(tracked['height']>median) & (tracked['time index'] == moment)]['ID'].unique()
    tracked = tracked.loc[tracked['ID'].isin(filtered_ids)]
    
    tracked_all = pd.DataFrame()
    for ID in tracked['ID'].unique():
        data = tracked.loc[tracked['ID']==ID]
        data.reset_index(inplace=True)
        subset = data[1:]
        non_zero_index = subset[(subset['dredged volume'] > 0) | (subset['ploughed_max'] > 0)].index
        if len(non_zero_index) > 0:
            first_non_zero_index = non_zero_index[0]
            data = data.iloc[:first_non_zero_index]
        dh = np.zeros(len(data))
        dh[0] = 0
        dh[1:] = np.diff(data['height'])
        dt = np.zeros(len(data))
        dt[0] = 0
        dt[1:] = np.diff(data['time']) / pd.Timedelta(days=1) 
        data['dh'] = dh
        data['dt'] = dt
        growth = dh/dt
        growth[0] = 0
        data['growth'] = growth
        data['dt0'] = (data['time'] - data['time'].iloc[0]) / pd.Timedelta(days=1)
        tracked_all = pd.concat([tracked_all, data], axis=0)
    tracked_all.dropna(inplace = True)
    tracked_all.reset_index(inplace=True)
    id_counts = tracked_all['ID'].value_counts()
    ids_to_keep = id_counts[id_counts >= interval].index
    tracked_all = tracked_all[tracked_all['ID'].isin(ids_to_keep)]
    return tracked_all

ID_counter = 0
tracked_all = pd.DataFrame()
for moment in range(len(file_info)):

    print(moment)
    try:
        tracked = createTrackedAll(moment, 2)
        tracked = tracked.reset_index(drop=True)
    except FileNotFoundError:
        continue
    except KeyError:
        print('Dataframe is empty')
        continue

    disch = np.zeros(len(tracked))
    for i in range(len(tracked)):
        
        disch[i] = file_info[file_info['time index'] == tracked.loc[i, 'time index']]['Q5'].values[0]
    tracked['Q5'] = disch
    
    for j, ID in enumerate(tracked['ID'].unique()):
        data = tracked.loc[tracked['ID']==ID]
        data = data.reset_index(drop=True)
        data['H0'] = data.loc[0, 'height']
        data['L0'] = data.loc[0, 'length']
        data['C0'] = data.loc[0, 'celerity XY']
        data['dredged_vol'] = data.loc[0, 'dredged volume']
        data['dune_vol'] = data.loc[0, 'dune volume']
        data['dredged_h'] = data.loc[0, 'dredged height']
        data['plough_sum'] = data.loc[0, 'ploughed_sum']
        data['plough_max'] = data.loc[0, 'ploughed_max']

        
        dq = np.zeros(len(data))
        for k in range(len(data)):
            if k == 0: dq[k] = 0
            else: dq[k] = data.loc[k, 'Q5'] - data.loc[0, 'Q5']
        data['dQ5'] = dq
        data['mainID'] = ID_counter
        ID_counter += 1
        
        tracked_all = pd.concat([tracked_all, data])

tracked_all['dQ5/dt'] = tracked_all['dQ5'] / tracked_all['dt0']
tracked_all['dQ5/dt'].fillna(0, inplace=True)

figname = hotspot + '_TA_ploeg_new2.pkl'
tracked_all.to_pickle(os.path.join(out_path, 'DevOverTime', figname))

#%%
main_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut/MainParameters'

def interpolate_nan_values(arr):
    # Find the indices of the non-NaN values
    non_nan_indices = np.where(~np.isnan(arr))[0]

    # Interpolate the NaN values between the first and last non-NaN values
    interpolated_arr = np.full_like(arr, np.nan)
    interpolated_arr[non_nan_indices[0]:non_nan_indices[-1] + 1] = np.interp(
        np.arange(non_nan_indices[0], non_nan_indices[-1] + 1),
        non_nan_indices,
        arr[non_nan_indices]
    )

    return interpolated_arr

df0=tracked_all.copy()


df0 = df0.loc[df0['dt0']<28]
df0 = df0.loc[df0['dt0']>0]


df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))

fig, axes = plt.subplots(2,1, figsize=(10,8), sharex=True, sharey=True)

ax = axes[0]
Y = 'H/H0'

perc5 = np.full((len(df0['time'].unique()),), np.nan)
perc25 = np.full((len(df0['time'].unique()),), np.nan)
perc75 = np.full((len(df0['time'].unique()),), np.nan)
perc95 = np.full((len(df0['time'].unique()),), np.nan)

median = np.full((len(df0['time'].unique()),), np.nan)

for i, time in enumerate(df0['time'].unique()):
    ax.axvline(time, color='Silver', linewidth=0.5, linestyle= '--')
    axes[1].axvline(time, color='Silver', linewidth=0.5, linestyle= '--')
    data = df0.loc[df0['time']==time]
    # data = data.loc[(data['hopper%']>=intervala[itv]) & (data['hopper%']<intervalb[itv])]
    if len(data) > 10:
        median[i] = data[Y].median() 
        perc5[i] = data[Y].quantile(0.10)
        perc25[i] = data[Y].quantile(0.25)
        perc75[i] = data[Y].quantile(0.75)
        perc95[i] = data[Y].quantile(0.90)

perc5 = interpolate_nan_values(perc5)
perc25 = interpolate_nan_values(perc25)
perc75 = interpolate_nan_values(perc75)
perc95 = interpolate_nan_values(perc95)
median = interpolate_nan_values(median) 
  
ax.scatter(df0['time'].unique(), median, color='black', s=10)
ax.plot(df0['time'].unique(), perc5, color='orange', linewidth=0.3)
ax.plot(df0['time'].unique(), perc25, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc75, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc95, color='orange', linewidth=0.3)
    
ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color='orange', label='50% confidence interval')
ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color='orange', label='90% confidence interval')
ax.legend(loc='upper center', fontsize = 'small')

ax.axhline(1)

dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax2 = ax.twinx()
# Plot the scatter plot on the right side axis
ax2.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax2.set_ylabel('discharge [m3/s]', color='grey')

ax.set_xlim([df0['time'].unique().min(), df0['time'].unique().max()])

ax.set_ylabel(Y)
ax.set_title('Ploughed dunes')
ax.set_ylim([0,2])


def getGrowthFactor(arr):
    arr1 = arr[:-1]
    arr2 = arr[1:]
    garr = arr2/arr1
    return garr

ax = axes[1]

name = hotspot + 'fileinfo.pkl'
file_info = pd.read_pickle(os.path.join(main_path, name))

file_info = file_info[file_info['TP Value'] == 0]
file_info = file_info.reset_index(drop=True)

dffname = hotspot + 'dff.pkl'
dff = pd.read_pickle(os.path.join(main_path, dffname))

plots = ['height']

timesteps = dff['time index'].unique()
for i, plot in enumerate(plots):
    perc5 = np.full((len(timesteps),), np.nan)
    perc25 = np.full((len(timesteps),), np.nan)
    perc50 = np.full((len(timesteps),), np.nan)
    perc75 = np.full((len(timesteps),), np.nan)
    perc95 = np.full((len(timesteps),), np.nan)
    median = np.full((len(timesteps),), np.nan)

    for j, ts in enumerate(timesteps):
        if hotspot == 'H' and ts in [14,15,16,17,18]:
            continue

        data = dff.loc[dff['time index'] == ts][plot]
        if plot =='celerity XY':
            data = data/7
        if len(data) == 0:
            continue
        boxplot_values = np.percentile(data, [10, 25, 50, 75, 90])
        median[j] = data.mean() 
        perc5[j] = data.quantile(0.10)
        perc25[j] = data.quantile(0.25)
        perc75[j] = data.quantile(0.75)
        perc95[j] = data.quantile(0.90)

    perc5 = interpolate_nan_values(perc5)
    perc25 = interpolate_nan_values(perc25)
    perc75 = interpolate_nan_values(perc75)
    perc95 = interpolate_nan_values(perc95)
    median = interpolate_nan_values(median)

    perc5 = getGrowthFactor(perc5)
    perc25 = getGrowthFactor(perc25)
    perc75 = getGrowthFactor(perc75)
    perc95 = getGrowthFactor(perc95)
    median = getGrowthFactor(median)

    time = dff['time'].unique()
    ax.axhline(1)

    ax.fill_between(time[1:], perc75, perc25, alpha=0.3, color='orange', label='50% confidence interval')
    ax.fill_between(time[1:], perc95, perc5, alpha=0.3, color='orange', label='90% confidence interval')
    ax.scatter(time[1:], median, color='black', label='Median', s=10)



# Create a twin Axes for the scatter plot
ax2 = ax.twinx()
# Plot the scatter plot on the right side axis
ax2.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax2.set_ylabel('discharge [m3/s]', color='grey')
        

ax.set_xlabel('Q$_{5\mathrm{days}}$ [m$^3$/s]')

ax.set_xlim([df0['time'].unique().min(), df0['time'].unique().max()])

ax.set_ylabel(Y)
ax.set_title('unaffected dunes')





#%% PLOT ALL
import math

def func(x, a):
    return a * x +1





rows = 4
cols = 2

df0 = tracked_all.copy()

df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['L0']
df0['c'] = df0['celerity XY']/7
# df0['lee angle'] = tracked_all['lee angle'] * -1

df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))

x = ['dt0', 'dQ5']

y = ['H/H0', 'L/L0', 'C/C0', 'c']

cmaps = ['Blues', 'Oranges', 'Greens', 'Reds', 'Greys']

def remove_outliersX(df, min_X, max_X):
    df = df.reset_index(drop=True)
    df = df.loc[(df['X']>=min_X) & (df['X']<=max_X)]
    df = df.reset_index(drop=True)
    return df

def remove_outliersY(df):
    df = df.reset_index(drop=True)
    df_filtered = pd.DataFrame()
    for i in df['X'].unique():
        dff = df.loc[df['X']==i]
        perc5 = dff['Y'].quantile(0.05)
        perc95 = dff['Y'].quantile(0.95)
        dff = dff.loc[(dff['Y']>=perc5) & (dff['Y']<=perc95)]
        df_filtered = pd.concat([df_filtered, dff])
    df_filtered = df_filtered.reset_index(drop=True)
    return df_filtered

dqwindow = 750

df0 = df0.loc[df0['c']<8]
df0 = df0.loc[df0['C0']<8*7]
# df0 = df0.loc[(df0['dQ5']>-dqwindow) & (df0['dQ5']<dqwindow)]


# df0 = df0.reset_index(drop=True)
# dft0 = df0.loc[df0['dt0'] == 0]
# lower = dft0['celerity XY'].quantile(0.1)
# upper = dft0['celerity XY'].quantile(0.8)
# dft0 = dft0.loc[(dft0['celerity XY'] > lower) & (dft0['celerity XY'] < upper)]
# mainIDs = dft0['mainID'].unique()
# df0 = df0.loc[df0['mainID'].isin(mainIDs)]

fig, ax = plt.subplots(rows,cols, figsize=(8,8), sharex='col', sharey='row')

# Z = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
Z= df0['plough_max']


for row in range(rows):
    Y = df0[y[row]]
    for col in range(cols):
        
        X = df0[x[col]]
        
        # Remove Outliers
        data = {'X': X, 'Y': Y, 'Z': Z}
        data = pd.DataFrame(data)
        data = remove_outliersY(data)
        
        ax[row,col].scatter(data['X'],data['Y'],c=data['Z'],cmap=cmaps[row],s=1)            
        ax[row,col].grid()
  

hopper_perc = [0, 3, 7]
linestyles = ['-', '--', ':', '-.']

for i, hp in enumerate(hopper_perc):      
    df1 = df0.loc[Z>hp] 
    df1 = df1.reset_index(drop=True)    
    for row in range(rows):
        Y = df1[y[row]]
        
        for col in range(cols):
            X = df1[x[col]]   
            
            # Remove Outliers
            data = {'X': X, 'Y': Y}
            data = pd.DataFrame(data)
            if col == 2:
                data = remove_outliersX(data, -30, 30)
            data = remove_outliersY(data)
            label = 'Plough intensity > ' + str(hp)
            if col == 0 and row in [5]:
                #plot trendline through a point
                xdata = data['X']
                ydata = data['Y']
                # Fit the curve
                popt, pcov = curve_fit(func, xdata, ydata)
                # Compute the predicted y-values using the fitted curve
                y_pred = func(xdata, *popt)
                # Create an extended x-range from 0 to 30
                x_range = np.linspace(xdata.min(), xdata.max(), 100)  # Adjust the number of points as needed
                # Plot the fitted curve
                # label = str(interval[i] * 100) + '-' + str(interval[i + 1] * 100) + '%, R2=' + r2_str
                ax[row,col].plot(x_range, func(x_range, *popt), linestyle=linestyles[i], color='black', linewidth=1, label=label)
            else:
                # Calculate and plot the trend line using all the data points in the subplot
                trend = np.polyfit(data['X'], data['Y'], 1)
                trendline = np.poly1d(trend)           
                # Generate x-values that span the range of the window
                xtrend = np.linspace(data['X'].min(), data['X'].max(), 100) 
                # Plot the trend line using the generated x-values
                ax[row,col].plot(xtrend, trendline(xtrend), linestyle=linestyles[i], color='black', linewidth=1, label=label)

ax[0,0].legend(loc='upper left', fontsize='small')

# Set axis labels
ax[3,0].set_xlabel('time [days]')
ax[3,1].set_xlabel('dQ5 [m3/s/day]')

ax[0,0].set_ylabel('H/H0 [-]')
ax[1,0].set_ylabel('L/L0 [-]')
ax[2,0].set_ylabel('C/C0 [-]')
ax[3,0].set_ylabel('C [m/day]')

# # Set axis limits
ax[2,0].set_xlim([0, 60])
ax[2,1].set_xlim([-dqwindow, dqwindow])

ax[0,0].set_ylim([0, 3])
ax[1,0].set_ylim([0, 3])
ax[2,0].set_ylim([0, 3])
ax[3,0].set_ylim([1, 9])


figname = 'HS-' + hotspot + '_Swiping.png'
fig.suptitle(figname, fontsize=16, fontweight='bold')
plt.tight_layout()
plt.savefig(os.path.join(out_path, 'DevOverTime', figname), dpi=300)