# -*- coding: utf-8 -*-
"""
Created on Mon Apr 24 14:04:21 2023

@author: kruis
"""
#%% Imports and functions

import os
import sys
import pandas as pd
import numpy as np
import json
import math

# add stream normal folder to sys path
sys.path.append('C:/Users/kruis/Documents/Thesis/Python/py_streamNormalcoords-master/')

# Import stream normal functions
import streamNormal as strNorm

def strCoords2Grid(combined, ds_min, ds_max, xs_min, xs_max):
    Zmax = np.full((xs_max - xs_min + 1, ds_max - ds_min + 1), np.nan)
    Zmin = np.full((xs_max - xs_min + 1, ds_max - ds_min + 1), np.nan)
    for row in range(len(combined)):
        xs1 = math.floor(combined.loc[row, 'xs'] - xs_min)
        ds1 = math.floor(combined.loc[row, 'ds'] - ds_min)
        z1 = combined.loc[row, 'Z']
        if xs1 < 0 or xs1 > xs_max-xs_min or ds1 < 0 or ds1 > ds_max - ds_min:
            continue
        elif not np.isnan(Zmax[xs1,ds1]) and not np.isnan(Zmin[xs1,ds1]):
            # Picks the heighest values at the tops of dunes
            Zmax[xs1,ds1] = max(Zmax[xs1,ds1], z1)   
            # Picks the lowest values at the throughs of dunes
            Zmin[xs1,ds1] = min(Zmin[xs1,ds1], z1)
        else:
            Zmax[xs1,ds1] = z1
            Zmin[xs1,ds1] = z1
    # interpolate the empty grid cells based on the average of the surrounding grid cells
    nan_count = np.count_nonzero(np.isnan(Zmax))
    print ("There are", nan_count, "NaN cells in the array.")
    Zmax = interpolate(Zmax)
    Zmin = interpolate(Zmin)
    nan_count = np.count_nonzero(np.isnan(Zmax))
    print (nan_count, "NaN cells remain after interpolation.")
    
    return Zmax, Zmin

def interpolate(arr):
    array = arr.copy()
    for i in range(array.shape[0]):
        for j in range(array.shape[1]):
            if np.isnan(array[i,j]):
                neighbors = []
                if i > 0 and not np.isnan(array[i-1,j]):
                    neighbors.append(array[i-1,j])
                if i < array.shape[0]-1 and not np.isnan(array[i+1,j]):
                    neighbors.append(array[i+1,j])
                if j > 0 and not np.isnan(array[i,j-1]):
                    neighbors.append(array[i,j-1])
                if j < array.shape[1]-1 and not np.isnan(array[i,j+1]):
                    neighbors.append(array[i,j+1])
                if len(neighbors) >= 3:
                    array[i,j] = np.mean(neighbors)
    return array

#%% Settings and load data

# Set search margins
# indicates the distance both up and downstream, 
# such that all points are in the search field in DS direction
margin = 200
search_width = 150

# Define string to determine hotspot index
hs_string = 'ABCDEFGHIJ'

# Dataframe with info about all raw data files
file_info_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES/file_info.pkl'
file_info_all = pd.read_pickle(file_info_path)

# Dataframe with info about the hotspots
hotspot_data = pd.read_excel('C:/Users/kruis/Documents/Thesis/Python/hotspot_data.xlsx')

# Dataframe with all points of the river Waal
# Load the river axis center line data 
river_all = pd.read_csv('C:/Users/kruis/Documents/pointscorda.csv')
# use boolean indexing to filter out rows that do not contain "WA" (Waal) in the 'state' column
river_all = river_all[river_all['MODELKILOM'].str.contains("WA")]
# drop irrelevant columns 
river_all.drop('OBJECTID', axis=1, inplace=True)
river_all.drop('MODELKILOM', axis=1, inplace=True)
river_all.drop('KENMERK', axis=1, inplace=True)

# path to folder where DataFrames and dictionaries can be stored
outpath = r'C:\Users\kruis\Documents\Thesis\Python\StreamCoords'
 
#%% Loop over all hotspots
for hotspot in hotspot_data['Hotspot']:
    
    print(hotspot)
    
    # Get hotspot index (potentially enumerate loop)
    index = hs_string.find(hotspot)
    
    # Determin the minimum and maximum X coordinate of the hotspot
    xmin, xmax = hotspot_data.loc[index, 'xmin'], hotspot_data.loc[index, 'xmax']
    
    # Update the Dataframe with file info to only contain files from the current hotspot
    file_info = file_info_all[file_info_all['Hotspot Letter'] == hotspot]
    file_info.reset_index(inplace=True, drop=True)
    
    # Create a list with all points, throughout all MBES measurements that should be transformed to stream coords
    points = pd.DataFrame(columns=['X', 'Y', 'Z'])
    # Loop over all file names. Tries to add the points to a dataframe. If not, file is 'corrupt', skip file
    for name in file_info['File Name']:
        try:
            file = pd.read_csv(os.path.join('D:/_metingen_DTON',name), header=None)
            file.columns = ['X', 'Y', 'Z']
            file = file.astype(int)
            points = pd.concat([points, file], axis=0, ignore_index=True)
        except ValueError: 
            print('Value Error')
            continue
    # Remove irrelevant info and duplicates, while ensuring the points are integers
    points.drop('Z', axis=1, inplace=True)
    points = points.astype(int)
    points = points.drop_duplicates()
    points = points.drop(points[points.X < xmin - margin].index, axis=0)
    points = points.drop(points[points.X > xmax + margin].index, axis=0)
    points.reset_index(drop=True, inplace=True)
    
    # Drop sections of the river outside of the hotspot to speed up calculations
    river = river_all
    river = river.drop(river[river.X < xmin - margin].index, axis=0)
    river = river.drop(river[river.X > xmax + margin].index, axis=0)
    river.reset_index(drop=True, inplace=True)
    
    # Transform XY coords to stream coords
    sn, clOut = strNorm.xy2sn(points.X,points.Y,river.X,river.Y,search_width,0)
    sn['X'] = sn['X'].astype(int)
    sn['Y'] = sn['Y'].astype(int)
    
    # Store the resulting DataFrames in pickle files
    name_sn = 'HS_' + hotspot + '_sn.pkl'
    name_cl = 'HS_' + hotspot + '_cl.pkl'
    sn.to_pickle(os.path.join(outpath,name_sn))
    clOut.to_pickle(os.path.join(outpath,name_cl))
    
#%% Transform all MBES files towards a 1x1 grid in Stream Coordinates  
    
    # Create two empty dictionaries which stores the maximum and minium elevation at each coordinate for each file
    SC_dict_max = {}
    SC_dict_min = {}
    
    # Define minimum and maximum values of the Stream Coords grid
    ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
    xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())

    for row in range(len(file_info)):
        print(row)
        # Load the MBES data and put in a dataframe
        file_name = os.path.join('D:/_metingen_DTON',file_info.loc[row, 'File Name'])

        # Processing
        try:
            mbes = pd.read_csv(file_name, header = None)
            mbes.columns = ['X', 'Y', 'Z']
            mbes['X'] = mbes['X'].astype(int)
            mbes['Y'] = mbes['Y'].astype(int)
            
            combined = pd.merge(sn, mbes, on=['X', 'Y'])
            combined.dropna(subset=['xs'], inplace=True)
            combined.dropna(subset=['ds'], inplace=True)
            combined.reset_index(inplace=True, drop=True)
    
            Zmax, Zmin = strCoords2Grid(combined, ds_min, ds_max, xs_min, xs_max)
        except ValueError:
            print('a value error occured')
            continue
        else:
            # Add to dictionary
            SC_dict_max.update({file_info.loc[row, 'Date Time']: Zmax})
            SC_dict_min.update({file_info.loc[row, 'Date Time']: Zmin})
          
    # Convert datetime keys and numpy arrays to serializable format
    serialized_dict = {}
    for key, value in SC_dict_max.items():
        serialized_dict[str(key)] = value.tolist()
    
    file_name = 'HS_' + hotspot + '_max.json'
    
    # Open a file for writing
    with open(os.path.join(outpath,file_name), 'w') as f:
        # Write the dictionary to the file in JSON format
        json.dump(serialized_dict, f)
    
    # Convert datetime keys and numpy arrays to serializable format
    serialized_dict = {}
    for key, value in SC_dict_min.items():
        serialized_dict[str(key)] = value.tolist()
    
    file_name = 'HS_' + hotspot + '_min.json'
    
    # Open a file for writing
    with open(os.path.join(outpath,file_name), 'w') as f:
        # Write the dictionary to the file in JSON format
        json.dump(serialized_dict, f)  
    

    
    
    
    
    
    