# -*- coding: utf-8 -*-
"""
Created on Sun Jun 18 20:50:18 2023

@author: kruis
"""

#%% INITIATE: 
    
hotspot = 'F'

# wdsmin, wdsmax, wxsmin, wxsmax = 150, 2200, 60, 210          # HS E
wdsmin, wdsmax, wxsmin, wxsmax = 300, 2200, 50, 200         # HS F
# wdsmin, wdsmax, wxsmin, wxsmax = 800, 2800, 0, 280          # HS - H
# wdsmin, wdsmax, wxsmin, wxsmax = 500, 1400, 65, 190         # HS J

dataframe_name = 'HS_' + hotspot + '_all_data.pkl'
# dataframe_name = 'HS_' + hotspot + '_all_data_rerun.pkl'
# dataframe_name = 'HS_' + hotspot + '_ws_small_all_data.pkl'    
    
import os
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import math
from datetime import datetime, timedelta, date
from scipy.optimize import curve_fit

import warnings
warnings.filterwarnings("ignore")

def interpolate_nan_values(arr):
    # Find the indices of the non-NaN values
    non_nan_indices = np.where(~np.isnan(arr))[0]

    # Interpolate the NaN values between the first and last non-NaN values
    interpolated_arr = np.full_like(arr, np.nan)
    interpolated_arr[non_nan_indices[0]:non_nan_indices[-1] + 1] = np.interp(
        np.arange(non_nan_indices[0], non_nan_indices[-1] + 1),
        non_nan_indices,
        arr[non_nan_indices]
    )

    return interpolated_arr

def get_previous_5days(date):
    # Create an empty array to store the result
    previous_days = []

    # Iterate over the range of 5 days
    for i in range(1, 6):
        # Subtract i days from the original date
        previous_day = date - timedelta(days=i)

        # Set the hours and minutes to 0
        previous_day = previous_day.replace(hour=0, minute=0)

        # Add the modified date to the result array
        previous_days.append(previous_day)

    return previous_days

out_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut'
StrCo_path = r'C:/Users/kruis/Documents/Thesis/Python/StreamCoords'

df_all = pd.read_pickle(os.path.join(out_path,dataframe_name))
df_all = df_all.loc[df_all['celerity XY'] < 100]
df_all['dredged perc'] = df_all['dredged volume'] / (df_all['dune volume'] + df_all['dredged volume'])
# Calculate the average 'celerity XY' per time index
average_celerity = df_all.groupby('time index')['celerity XY'].mean()
# Create a new column 'average_celerity' in the DataFrame
df_all['average celerity'] = df_all['time index'].map(average_celerity)

# file_info_name = 'HS_'+hotspot+'_FI_test.pkl'
file_info_name = 'HS_'+hotspot+'_FI.pkl'
file_info = pd.read_pickle(os.path.join(out_path,file_info_name))  
file_info = file_info.reset_index(drop=True)
file_info['time index'] = file_info.index

# get transformed sn file
sn_name = 'HS_' + hotspot + '_sn.pkl'
sn = pd.read_pickle(os.path.join(StrCo_path,sn_name))

# Define minimum and maximum values of the Stream Coords grid
ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())

# import discharge data
Q = pd.read_csv(r'C:/Users/kruis/Documents/Thesis/files_RWS/afvoeren/Pan.csv', delimiter=';')
Q = Q[['datum', 'tijd', 'waarde']]
Q['waarde'] = Q['waarde'].str.replace(',', '.').astype(float)
Q['datum'] = Q['datum'] + ' ' + Q['tijd']
Q['datum'] = pd.to_datetime(Q['datum'], format='%d-%m-%Y %H:%M')
Q = Q.loc[Q['waarde']<10000]
Q = Q.loc[Q['waarde']>0]

# Group the DataFrame by date and calculate the average 'waarde' per day
Q_daily = Q.groupby(Q['datum'].dt.date)['waarde'].mean().reset_index()

# Rename the columns
Q_daily.columns = ['date', 'average_waarde']

# Convert the 'date' column back to datetime format
Q_daily['date'] = pd.to_datetime(Q_daily['date'])

discharge5 = np.zeros(len(file_info))
discharge = np.zeros(len(file_info))
for i, row in file_info.iterrows():
    time = row['Date Time']
    past5days = get_previous_5days(time)
    dfQ5 = Q_daily.loc[Q_daily['date'].isin(past5days)]
    dfQ = Q_daily.loc[Q_daily['date'].dt.date == file_info.loc[i, 'Date Time'].date()]
    discharge[i] = dfQ['average_waarde'].values[0]
    discharge5[i] = dfQ5['average_waarde'].mean() 
file_info['Q'] = discharge
file_info['Q5'] = discharge5

# Remove values outside the specified range
df_filtered = df_all[(df_all['ds'] >= wdsmin) & (df_all['ds'] <= wdsmax) & (df_all['xs'] >= wxsmin) & (df_all['xs'] <= wxsmax)]

extent = [wdsmax - wdsmin, 0, wxsmin+xs_min, wxsmax+xs_min]

# Remove TP values and measurements to close to each other
file_info_filtered = file_info[file_info['TP Value'] == 0]
time_steps = file_info_filtered['time index'].unique()
dff = df_all.loc[df_all['time index'].isin(file_info_filtered['time index'])]

# determine average growth for non dredged areas
overall = dff.loc[(df_all['dredged perc'] == 0) & (df_all['ploughed_max'] ==0)]
overall_mean_h = overall.groupby('time index')['height'].median()
overall_mean_dh = np.diff(np.array(overall_mean_h))
overall_mean_T = overall.groupby('time index')['time'].median()
overall_mean_T = (overall_mean_T - overall_mean_T.min()).dt.total_seconds() / (24 * 60 * 60)
overall_mean_dT = np.diff(np.array(overall_mean_T))
overall_mean_growth = overall_mean_dh / overall_mean_dT
overall_mean_growth_rel = overall_mean_dh / overall_mean_h[:-1] / overall_mean_dT
overall_mean_growth_rel = pd.DataFrame(overall_mean_growth_rel, columns=['height'])
overall_mean_growth_rel = overall_mean_growth_rel.rename(columns={'height': 'growth'})
overall_mean_growth_rel['growth'] = overall_mean_growth

file_info = file_info.merge(overall_mean_growth_rel, how='left', left_on='time index', right_index=True)


#%% PREPROCESS: tracked height no dredging multiple

def createTrackedAll(moment, interval):
    trackname = str(moment) + '.pkl'
    tracked = pd.read_pickle(os.path.join(out_path,hotspot,trackname))
    tracked = tracked.drop('Z', axis=1)
    tracked = tracked.drop('migration angle', axis=1)
    median = tracked.loc[(tracked['time index'] == moment)]['height'].median()
    median = 0.2
    filtered_ids = tracked.loc[(tracked['height']>median) & (tracked['time index'] == moment)]['ID'].unique()
    tracked = tracked.loc[tracked['ID'].isin(filtered_ids)]
    
    tracked_all = pd.DataFrame()
    for ID in tracked['ID'].unique():
        data = tracked.loc[tracked['ID']==ID]
        data.reset_index(inplace=True)
        subset = data[1:]
        non_zero_index = subset[(subset['dredged volume'] > 0) | (subset['ploughed_max'] > 0)].index
        if len(non_zero_index) > 0:
            first_non_zero_index = non_zero_index[0]
            data = data.iloc[:first_non_zero_index]
        dh = np.zeros(len(data))
        dh[0] = 0
        dh[1:] = np.diff(data['height'])
        dt = np.zeros(len(data))
        dt[0] = 0
        dt[1:] = np.diff(data['time']) / pd.Timedelta(days=1) 
        data['dh'] = dh
        data['dt'] = dt
        growth = dh/dt
        growth[0] = 0
        data['growth'] = growth
        data['dt0'] = (data['time'] - data['time'].iloc[0]) / pd.Timedelta(days=1)
        tracked_all = pd.concat([tracked_all, data], axis=0)
    tracked_all.dropna(inplace = True)
    tracked_all.reset_index(inplace=True)
    id_counts = tracked_all['ID'].value_counts()
    ids_to_keep = id_counts[id_counts >= interval].index
    tracked_all = tracked_all[tracked_all['ID'].isin(ids_to_keep)]
    return tracked_all

ID_counter = 0
tracked_all = pd.DataFrame()
for moment in range(len(file_info)):

    print(moment)
    try:
        tracked = createTrackedAll(moment, 2)
        tracked = tracked.reset_index(drop=True)
    except FileNotFoundError:
        continue
    except KeyError:
        print('Dataframe is empty')
        continue

    disch = np.zeros(len(tracked))
    for i in range(len(tracked)):
        try:
            disch[i] = file_info[file_info['time index'] == tracked.loc[i, 'time index']]['Q5'].values[0]
        except IndexError:
            continue
        
    tracked['Q5'] = disch
    
    for j, ID in enumerate(tracked['ID'].unique()):
        data = tracked.loc[tracked['ID']==ID]
        data = data.reset_index(drop=True)
        data['H0'] = data.loc[0, 'height']
        data['L0'] = data.loc[0, 'length']
        data['C0'] = data.loc[0, 'celerity XY']
        data['dredged_vol'] = data.loc[0, 'dredged volume']
        data['dune_vol'] = data.loc[0, 'dune volume']
        data['dredged_h'] = data.loc[0, 'dredged height']
        data['plough_sum'] = data.loc[0, 'ploughed_sum']
        data['plough_max'] = data.loc[0, 'ploughed_max']

        
        dq = np.zeros(len(data))
        for k in range(len(data)):
            if k == 0: dq[k] = 0
            else: dq[k] = data.loc[k, 'Q5'] - data.loc[0, 'Q5']
        data['dQ5'] = dq
        data['mainID'] = ID_counter
        ID_counter += 1
        
        tracked_all = pd.concat([tracked_all, data])

tracked_all['dQ5/dt'] = tracked_all['dQ5'] / tracked_all['dt0']
tracked_all['dQ5/dt'].fillna(0, inplace=True)


figname = hotspot + '_TA_hop_new.pkl'
tracked_all.to_pickle(os.path.join(out_path, 'DevOverTime', figname))


#%%

figname ='E_TA_hop.pkl'
E = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='F_TA_hop.pkl'
F = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='h_TA_hop.pkl'
H = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))

# df0 = pd.concat([E,F,H])

df0 = H
# df0 = tracked_all.copy()

main_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut/MainParameters'

def interpolate_nan_values(arr):
    # Find the indices of the non-NaN values
    non_nan_indices = np.where(~np.isnan(arr))[0]

    # Interpolate the NaN values between the first and last non-NaN values
    interpolated_arr = np.full_like(arr, np.nan)
    interpolated_arr[non_nan_indices[0]:non_nan_indices[-1] + 1] = np.interp(
        np.arange(non_nan_indices[0], non_nan_indices[-1] + 1),
        non_nan_indices,
        arr[non_nan_indices]
    )

    return interpolated_arr

df0 = df0.loc[df0['dt0']<14]
df0 = df0.loc[df0['dt0']>0]


df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))


df0['H/H0/dt0'] = df0['H/H0']**(1/df0['dt0'])

fig, axes = plt.subplots(2,1, figsize=(10,8), sharex=True, sharey=True)

ax = axes[0]
Y = 'H/H0/dt0'

perc5 = np.full((len(df0['time'].unique()),), np.nan)
perc25 = np.full((len(df0['time'].unique()),), np.nan)
perc75 = np.full((len(df0['time'].unique()),), np.nan)
perc95 = np.full((len(df0['time'].unique()),), np.nan)

median = np.full((len(df0['time'].unique()),), np.nan)

for i, time in enumerate(df0['time'].unique()):
    ax.axvline(time, color='Silver', linewidth=0.5, linestyle= '--')
    axes[1].axvline(time, color='Silver', linewidth=0.5, linestyle= '--')
    data = df0.loc[df0['time']==time]
    # data = data.loc[(data['hopper%']>=intervala[itv]) & (data['hopper%']<intervalb[itv])]
    if len(data) > 10:
        median[i] = data[Y].median() 
        perc5[i] = data[Y].quantile(0.10)
        perc25[i] = data[Y].quantile(0.25)
        perc75[i] = data[Y].quantile(0.75)
        perc95[i] = data[Y].quantile(0.90)

perc5 = interpolate_nan_values(perc5)
perc25 = interpolate_nan_values(perc25)
perc75 = interpolate_nan_values(perc75)
perc95 = interpolate_nan_values(perc95)
median = interpolate_nan_values(median) 
  
ax.plot(df0['time'].unique(), perc5, color='orange', linewidth=0.3)
ax.plot(df0['time'].unique(), perc25, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc75, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc95, color='orange', linewidth=0.3)
    
ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color='orange', label='50% confidence interval')
ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color='orange', label='90% confidence interval')
ax.scatter(df0['time'].unique(), median, color='black', s=10)

ax.legend(loc='upper center', fontsize = 'small')

dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax2 = ax.twinx()
# Plot the scatter plot on the right side axis
ax2.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax2.set_ylabel('discharge [m3/s]', color='grey')

def getGrowthFactor(arr):
    arr1 = arr[:-1]
    arr2 = arr[1:]
    garr = arr2/arr1
    return garr

name = hotspot + 'fileinfo.pkl'
file_info = pd.read_pickle(os.path.join(main_path, name))

file_info = file_info[file_info['TP Value'] == 0]
file_info = file_info.reset_index(drop=True)

dffname = hotspot + 'dff.pkl'
dff = pd.read_pickle(os.path.join(main_path, dffname))

timesteps = dff['time index'].unique()

median = np.full((len(timesteps),), np.nan)
dt = np.full((len(timesteps)-1,), np.nan)
for j, ts in enumerate(timesteps):
    if hotspot == 'H' and ts in [14,15,16,17,18]:
        continue
    if not j == len(timesteps)-1:
        curr = dff.loc[dff['time index'] == timesteps[j]]['time'].unique()[0]
        nxt = dff.loc[dff['time index'] == timesteps[j+1]]['time'].unique()[0]
        diff = (nxt - curr).astype('timedelta64[s]').item()
        dt[j] = float(diff.total_seconds()) / (60*60*24)
    data = dff.loc[dff['time index'] == ts]['height']
    if len(data) == 0:
        continue
    median[j] = data.median() 

median = interpolate_nan_values(median)
median = getGrowthFactor(median)**(1/(dt))
time = dff['time'].unique()
ax.plot(time[1:], median, color='red', label='Median', linestyle='--', linewidth=0.8)
ax.axhline(1, linewidth=0.3)

ax.set_xlim([df0['time'].unique()[7], df0['time'].unique().max()])
ax.set_ylabel('Daily H/H0 [1/day]')
ax.set_title('Topping')
ax.set_ylim([0.9, 1.15])



















figname ='E_TA_ploeg.pkl'
E = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='F_TA_ploeg.pkl'
F = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='h_TA_ploeg.pkl'
H = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))

# df0 = pd.concat([E,F,H])

df0 = H

df0 = df0.loc[df0['dt0']<50]
df0 = df0.loc[df0['dt0']>0]


df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))


df0['H/H0/dt0'] = df0['H/H0']**(1/df0['dt0'])

ax = axes[1]
Y = 'H/H0/dt0'

perc5 = np.full((len(df0['time'].unique()),), np.nan)
perc25 = np.full((len(df0['time'].unique()),), np.nan)
perc75 = np.full((len(df0['time'].unique()),), np.nan)
perc95 = np.full((len(df0['time'].unique()),), np.nan)

median = np.full((len(df0['time'].unique()),), np.nan)

for i, time in enumerate(df0['time'].unique()):
    # ax.axvline(time, color='Silver', linewidth=0.5, linestyle= '--')
    # axes[1].axvline(time, color='Silver', linewidth=0.5, linestyle= '--')
    data = df0.loc[df0['time']==time]
    if len(data) > 10:
        median[i] = data[Y].median() 
        perc5[i] = data[Y].quantile(0.10)
        perc25[i] = data[Y].quantile(0.25)
        perc75[i] = data[Y].quantile(0.75)
        perc95[i] = data[Y].quantile(0.90)

perc5 = interpolate_nan_values(perc5)
perc25 = interpolate_nan_values(perc25)
perc75 = interpolate_nan_values(perc75)
perc95 = interpolate_nan_values(perc95)
median = interpolate_nan_values(median) 
  
ax.plot(df0['time'].unique(), perc5, color='orange', linewidth=0.3)
ax.plot(df0['time'].unique(), perc25, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc75, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc95, color='orange', linewidth=0.3)
    
ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color='orange', label='50% confidence interval')
ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color='orange', label='90% confidence interval')
ax.scatter(df0['time'].unique(), median, color='black', s=10)

ax.legend(loc='upper center', fontsize = 'small')

dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax2 = ax.twinx()
# Plot the scatter plot on the right side axis
ax2.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax2.set_ylabel('discharge [m3/s]', color='grey')

def getGrowthFactor(arr):
    arr1 = arr[:-1]
    arr2 = arr[1:]
    garr = arr2/arr1
    return garr

name = hotspot + 'fileinfo.pkl'
file_info = pd.read_pickle(os.path.join(main_path, name))

file_info = file_info[file_info['TP Value'] == 0]
file_info = file_info.reset_index(drop=True)

dffname = hotspot + 'dff.pkl'
dff = pd.read_pickle(os.path.join(main_path, dffname))

timesteps = dff['time index'].unique()

median = np.full((len(timesteps),), np.nan)
dt = np.full((len(timesteps)-1,), np.nan)
for j, ts in enumerate(timesteps):
    if hotspot == 'H' and ts in [14,15,16,17,18]:
        continue
    if not j == len(timesteps)-1:
        curr = dff.loc[dff['time index'] == timesteps[j]]['time'].unique()[0]
        nxt = dff.loc[dff['time index'] == timesteps[j+1]]['time'].unique()[0]
        diff = (nxt - curr).astype('timedelta64[s]').item()
        dt[j] = float(diff.total_seconds()) / (60*60*24)
    data = dff.loc[dff['time index'] == ts]['height']
    if len(data) == 0:
        continue
    median[j] = data.median() 

median = interpolate_nan_values(median)
median = getGrowthFactor(median)**(1/(dt))
time = dff['time'].unique()
ax.plot(time[1:], median, color='red', label='Median', linestyle='--', linewidth=0.8)
ax.axhline(1, linewidth=0.3)


ax.set_ylabel('Daily H/H0 [1/day]')
ax.set_title('Swiping')
ax.set_ylim([0.9, 1.15])













#%%

df0=tracked_all.copy()

df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))


# colors = ['#FFA500', '#4169E1', '#008000', 'silver', 'red']

colors = [
    '#FF0000',  # Shade 1
    # '#FF3333',  # Shade 2
    # '#FF6666',  # Shade 3
    
    # '#FF9999',  # Shade 4
    '#FFCCCC',   # Shade 5
    
    'purple'
]


def interpolate_nan_values(arr):
    # Find the indices of the non-NaN values
    non_nan_indices = np.where(~np.isnan(arr))[0]

    # Interpolate the NaN values between the first and last non-NaN values
    interpolated_arr = np.full_like(arr, np.nan)
    interpolated_arr[non_nan_indices[0]:non_nan_indices[-1] + 1] = np.interp(
        np.arange(non_nan_indices[0], non_nan_indices[-1] + 1),
        non_nan_indices,
        arr[non_nan_indices]
    )

    return interpolated_arr

# perc5 = interpolate_nan_values(perc5)
# perc25 = interpolate_nan_values(perc25)
# perc75 = interpolate_nan_values(perc75)
# perc95 = interpolate_nan_values(perc95)
# median = interpolate_nan_values(median) 


fig, axes = plt.subplots(2,1, figsize=(10,8))


Y = 'H/H0'


# perc5 = np.full((len(df0['time'].unique()),), np.nan)
# perc25 = np.full((len(df0['time'].unique()),), np.nan)
# perc50 = np.full((len(df0['time'].unique()),), np.nan)
# perc75 = np.full((len(df0['time'].unique()),), np.nan)
# perc95 = np.full((len(df0['time'].unique()),), np.nan)
median = np.full((len(df0['time'].unique()),), np.nan)

intervala = [0.1, 7, 14, 21, 28]
intervalb = [7,  14, 21, 28, 35]


intervala = [0.01, 0.03, 0.10]
intervalb = [0.03, 0.10, 0.20]

ax = axes[0]

for itv in range(len(intervala)):
    for i, time in enumerate(df0['time'].unique()):
        ax.axvline(time, color='k', linewidth=0.05, linestyle= '--')
        data = df0.loc[df0['time']==time]
        data = data.loc[(data['hopper%']>=intervala[itv]) & (data['hopper%']<intervalb[itv])]
        if len(data) > 5:
            median[i] = data[Y].median() 

    
    # ax.scatter(df0['time'].unique(),median)
    label = 'Hopper%: ' + str(intervala[itv]) + ' - ' + str(intervalb[itv])
    ax.scatter(df0['time'].unique(), median, color=colors[itv], label=label, s=10)
# ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color=colors[i], label='50% confidence interval')
# ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color=colors[i], label='90% confidence interval')
ax.legend(loc='upper center', fontsize = 'small')

ax.axhline(1)

dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax2 = ax.twinx()
# Plot the scatter plot on the right side axis
ax2.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax2.set_ylabel('discharge [m3/s]', color='grey')

ax.set_xlim([df0['time'].unique().min(), df0['time'].unique().max()])

ax.set_ylabel(Y)


ax = axes[1]
colors = [
    '#FF0000',  # Shade 1
    # '#FF3333',  # Shade 2
    '#FF6666',  # Shade 3
    # '#FF9999',  # Shade 4
    '#FFCCCC'   # Shade 5
]
intervala = [0.001, 7, 21]
intervalb = [7, 21, 35]
for itv in range(len(intervala)):
    for i, time in enumerate(df0['time'].unique()):
        ax.axvline(time, color='k', linewidth=0.05, linestyle= '--')
        data = df0.loc[df0['time']==time]
        data = data.loc[(data['dt0']>=intervala[itv]) & (data['hopper%']<intervalb[itv])]
        if len(data) > 5:
            median[i] = data[Y].median() 

    
    # ax.scatter(df0['time'].unique(),median)
    label = 'Days since dredging: ' + str(intervala[itv]) + ' - ' + str(intervalb[itv])
    ax.scatter(df0['time'].unique(), median, color=colors[itv], label=label, s=10)
# ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color=colors[i], label='50% confidence interval')
# ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color=colors[i], label='90% confidence interval')
ax.legend(loc='upper center', fontsize = 'small')

ax.axhline(1)

dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax2 = ax.twinx()
# Plot the scatter plot on the right side axis
ax2.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.plot(np.array(Q_daily['date'])[1:], np.diff(Q_daily['average_waarde']), color='grey')
# Set the label and color for the right side axis
ax2.set_ylabel('discharge diff [m3/s]', color='grey')

ax.set_xlim([df0['time'].unique().min(), df0['time'].unique().max()])
ax.set_ylim([0, 2])

# ax2.set_ylim([-600, 600])
ax.set_ylabel(Y)

#%% PLOT ALL
import math
from matplotlib import cm

def func(x, a):
    return a * x + 1

def remove_outliersX(df, min_X, max_X):
    df = df.reset_index(drop=True)
    df = df.loc[(df['X']>=min_X) & (df['X']<=max_X)]
    df = df.reset_index(drop=True)
    return df

def remove_outliersY(df):
    df = df.reset_index(drop=True)
    df_filtered = pd.DataFrame()
    for i in df['X'].unique():
        dff = df.loc[df['X']==i]
        perc5 = dff['Y'].quantile(0.05)
        perc95 = dff['Y'].quantile(0.95)
        dff = dff.loc[(dff['Y']>=perc5) & (dff['Y']<=perc95)]
        df_filtered = pd.concat([df_filtered, dff])
    df_filtered = df_filtered.reset_index(drop=True)
    return df_filtered




figname ='E_TA_hop.pkl'
E = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='F_TA_hop.pkl'
F = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='H_TA_hop_new.pkl'
H = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))

df0 = pd.concat([E,F,H])
df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged v/l'] = df0['dredged_vol'] / df0['L0']
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))

x = ['dt0']

y = ['H/H0', 'L/L0', 'C/C0']

cmaps = ['Blues', 'Oranges', 'Greens', 'Reds', 'Greys']
colors = ['red', 'green', 'blue', 'orange']



dqwindow = 750
rows = 3
cols = 2




import matplotlib.gridspec as gridspec

# gridspec inside gridspec
fig = plt.figure(figsize=(10,7))

gs0 = gridspec.GridSpec(1, 2, figure=fig)

gs00 = gridspec.GridSpecFromSubplotSpec(3, 2, subplot_spec=gs0[0])

# Create the subplots
ax1 = fig.add_subplot(gs00[-3, -2])
ax2 = fig.add_subplot(gs00[-3, -1])
ax3 = fig.add_subplot(gs00[-2, -2])
ax4 = fig.add_subplot(gs00[-2, -1])
ax5 = fig.add_subplot(gs00[-1, -2])
ax6 = fig.add_subplot(gs00[-1, -1])

# Set sharex and sharey for consistent scaling and alignment
ax1.sharex(ax4)
ax1.sharey(ax4)
ax2.sharex(ax4)
ax2.sharey(ax4)
ax3.sharex(ax4)
ax3.sharey(ax4)
ax5.sharex(ax4)
ax5.sharey(ax4)
ax6.sharex(ax4)
ax6.sharey(ax4)


# the following syntax does the same as the GridSpecFromSubplotSpec call above:
gs01 = gs0[1].subgridspec(2, 1)

ax7= fig.add_subplot(gs01[-2, :])
ax8 = fig.add_subplot(gs01[-1, :])

ax8.sharex(ax7)
ax8.sharey(ax7)
  
hopper_perc = [0.01, 0.1, 0.25, 0.5]
linestyles = ['-', '--', ':', '-.']

col = 0
filt = 'hopper%'

print(df0[filt].min())
print(df0[filt].max())

for i, hp in enumerate(hopper_perc): 
    if hp ==0.5: break    
    df1 = df0.loc[(df0[filt]>hp) & (df0[filt]<hopper_perc[i+1])] 
    df1 = df1.reset_index(drop=True)    
    for row in range(rows):
        
        if row == 0: ax = ax1
        elif row == 1: ax = ax3
        elif row == 2: ax = ax5
        
        Y = df1[y[row]]
        X = df1[x[col]]   
        
        # Remove Outliers
        data = {'X': X, 'Y': Y}
        data = pd.DataFrame(data)
        if col == 2:
            data = remove_outliersX(data, -30, 30)
        data = remove_outliersY(data)
        
        # Grouping by intervals of 5 for 'X' column
        max_x = int(max(data['X']))
        min_x = int(min(data['X']))
        
        if col == 0: 
            binsize = 10
        if col == 1: 
            binsize = 200
        
        data['X_intervals'] = pd.cut(data['X'], bins=range(min_x, max_x+binsize+1, binsize), right=False)
        group = data.groupby('X_intervals').mean()
        # group = group.dropna()
        
        label = str(hp*100) + '-' + str(hopper_perc[i+1]*100) + '%'
        
        ax.plot(group['X'],group['Y'],color=colors[i], label=label)              
        ax.grid()
            







figname ='E_TA_ploeg.pkl'
E = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='F_TA_ploeg.pkl'
F = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='H_TA_ploeg_new.pkl'
H = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))

df0 = pd.concat([E,F,H])
df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['L0'] = df0['L0'].astype(float)
df0['plough%'] = df0['plough_sum'] / df0['L0']
df0['c'] = df0['celerity XY']/7

df0['dune_vol'] = df0['dune_vol'].astype(float)



x = ['dt0','dt0']
y = ['H/H0', 'L/L0', 'C/C0', 'c']

ploeg_perc = [0, 2, 5, 10]

col = 1
filt = 'plough%'

print(df0[filt].min())
print(df0[filt].max())

for i, hp in enumerate(ploeg_perc): 
    print(hp)
    if hp == 10: break    
    
    df1 = df0.loc[(df0[filt]>hp) & (df0[filt]<ploeg_perc[i+1])] 
    df1 = df1.reset_index(drop=True)    
    for row in range(rows):
        if row == 0: ax = ax2
        elif row == 1: ax = ax4
        elif row == 2: ax = ax6
        
        Y = df1[y[row]]
        X = df1[x[col]]   
        
        # Remove Outliers
        data = {'X': X, 'Y': Y}
        data = pd.DataFrame(data)
        if col == 2:
            data = remove_outliersX(data, -30, 30)
        data = remove_outliersY(data)
        
        # Grouping by intervals of 5 for 'X' column
        max_x = int(max(data['X']))
        min_x = int(min(data['X']))
        
        if col == 0: 
            binsize = 10
        if col == 1: 
            binsize = 10
        
        data['X_intervals'] = pd.cut(data['X'], bins=range(min_x, max_x+binsize+1, binsize), right=False)
        group = data.groupby('X_intervals').mean()
        # group = group.dropna()
        
        label = str(hp) + '-' + str(ploeg_perc[i+1])
        
        ax.plot(group['X'],group['Y'],color=colors[i], label=label)              
        ax.grid()












figname ='E_TA_hop.pkl'
E = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='F_TA_hop.pkl'
F = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='H_TA_hop.pkl'
H = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))

# df0 = pd.concat([E,F,H])

df0 = H
# df0 = tracked_all.copy()

main_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut/MainParameters'

def interpolate_nan_values(arr):
    # Find the indices of the non-NaN values
    non_nan_indices = np.where(~np.isnan(arr))[0]

    # Interpolate the NaN values between the first and last non-NaN values
    interpolated_arr = np.full_like(arr, np.nan)
    interpolated_arr[non_nan_indices[0]:non_nan_indices[-1] + 1] = np.interp(
        np.arange(non_nan_indices[0], non_nan_indices[-1] + 1),
        non_nan_indices,
        arr[non_nan_indices]
    )

    return interpolated_arr

df0 = df0.loc[df0['dt0']<30]
df0 = df0.loc[df0['dt0']>0]


df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))


df0['H/H0/dt0'] = df0['H/H0']**(1/df0['dt0'])

# df0 = df0.loc[df0['hopper%']>0.01]

ax = ax7
Y = 'H/H0/dt0'

perc5 = np.full((len(df0['time'].unique()),), np.nan)
perc25 = np.full((len(df0['time'].unique()),), np.nan)
perc75 = np.full((len(df0['time'].unique()),), np.nan)
perc95 = np.full((len(df0['time'].unique()),), np.nan)

median = np.full((len(df0['time'].unique()),), np.nan)

for i, time in enumerate(df0['time'].unique()):
    ax.axvline(time, color='Silver', linewidth=0.2, linestyle= '-')
    ax8.axvline(time, color='Silver', linewidth=0.2, linestyle= '-')
    data = df0.loc[df0['time']==time]
    # data = data.loc[(data['hopper%']>=intervala[itv]) & (data['hopper%']<intervalb[itv])]
    if len(data) > 10:
        median[i] = data[Y].median() 
        perc5[i] = data[Y].quantile(0.10)
        perc25[i] = data[Y].quantile(0.25)
        perc75[i] = data[Y].quantile(0.75)
        perc95[i] = data[Y].quantile(0.90)

perc5 = interpolate_nan_values(perc5)
perc25 = interpolate_nan_values(perc25)
perc75 = interpolate_nan_values(perc75)
perc95 = interpolate_nan_values(perc95)
median = interpolate_nan_values(median) 
  
ax.plot(df0['time'].unique(), perc5, color='orange', linewidth=0.3)
ax.plot(df0['time'].unique(), perc25, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc75, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc95, color='orange', linewidth=0.3)
    
ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color='orange', label='50% confidence interval')
ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color='orange', label='90% confidence interval')
ax.scatter(df0['time'].unique(), median, color='black', s=10, label='Median')



dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax9 = ax.twinx()
# Plot the scatter plot on the right side axis
ax9.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax9.set_ylabel('discharge [m3/s]', color='grey')

def getGrowthFactor(arr):
    arr1 = arr[:-1]
    arr2 = arr[1:]
    garr = arr2/arr1
    return garr

name = hotspot + 'fileinfo.pkl'
file_info = pd.read_pickle(os.path.join(main_path, name))

file_info = file_info[file_info['TP Value'] == 0]
file_info = file_info.reset_index(drop=True)

dffname = hotspot + 'dff.pkl'
dff = pd.read_pickle(os.path.join(main_path, dffname))

timesteps = dff['time index'].unique()

median = np.full((len(timesteps),), np.nan)
dt = np.full((len(timesteps)-1,), np.nan)
for j, ts in enumerate(timesteps):
    if hotspot == 'H' and ts in [14,15,16,17,18]:
        continue
    if not j == len(timesteps)-1:
        curr = dff.loc[dff['time index'] == timesteps[j]]['time'].unique()[0]
        nxt = dff.loc[dff['time index'] == timesteps[j+1]]['time'].unique()[0]
        diff = (nxt - curr).astype('timedelta64[s]').item()
        dt[j] = float(diff.total_seconds()) / (60*60*24)
    data = dff.loc[dff['time index'] == ts]['height']
    if len(data) == 0:
        continue
    median[j] = data.median() 

median = interpolate_nan_values(median)
median = getGrowthFactor(median)**(1/(dt))
time = dff['time'].unique()
ax.plot(time[1:], median, color='red', label='Unaffected dunes (median)', linestyle='--', linewidth=1)
ax.axhline(1, linewidth=0.3)

ax.set_xlim([df0['time'].unique()[7], df0['time'].unique().max()])
ax.set_ylabel('Daily Growth [1/day]')
ax.set_title('Topping')
ax.set_ylim([0.9, 1.15])




ax.legend(loc='upper left', fontsize =9)














figname ='E_TA_ploeg.pkl'
E = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='F_TA_ploeg.pkl'
F = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))
figname ='H_TA_ploeg_new.pkl'
H = pd.read_pickle(os.path.join(out_path, 'DevOverTime', figname))

# df0 = pd.concat([E,F,H])

df0 = H

df0 = df0.loc[df0['dt0']<30]
df0 = df0.loc[df0['dt0']>0]


df0['H/H0'] = df0['height']/df0['H0'] 
df0['L/L0'] = df0['length']/df0['L0'] 
df0['C/C0'] = df0['celerity XY']/df0['C0']
df0['hopper%'] = df0['dredged_vol'] / (df0['dune_vol'] + df0['dredged_vol'])
df0['plough%'] = df0['plough_sum'] / df0['length']
df0['c'] = df0['celerity XY']/7
df0['L0'] = df0['L0'].astype(float)
df0['dune_vol'] = df0['dune_vol'].astype(float)
df0['dredged_vol'] = df0['dredged_vol'].astype(float)
df0['dredged calc h'] = 2 * df0['dredged_vol'] / (df0['L0'] / (df0['dune_vol'] / df0['dredged_vol']).apply(math.sqrt))


df0['H/H0/dt0'] = df0['H/H0']**(1/df0['dt0'])

# df0 = df0.loc[df0['plough%']>0.01]

ax = ax8
Y = 'H/H0/dt0'

perc5 = np.full((len(df0['time'].unique()),), np.nan)
perc25 = np.full((len(df0['time'].unique()),), np.nan)
perc75 = np.full((len(df0['time'].unique()),), np.nan)
perc95 = np.full((len(df0['time'].unique()),), np.nan)

median = np.full((len(df0['time'].unique()),), np.nan)

for i, time in enumerate(df0['time'].unique()):
    data = df0.loc[df0['time']==time]
    if len(data) > 10:
        median[i] = data[Y].median() 
        perc5[i] = data[Y].quantile(0.10)
        perc25[i] = data[Y].quantile(0.25)
        perc75[i] = data[Y].quantile(0.75)
        perc95[i] = data[Y].quantile(0.90)

perc5 = interpolate_nan_values(perc5)
perc25 = interpolate_nan_values(perc25)
perc75 = interpolate_nan_values(perc75)
perc95 = interpolate_nan_values(perc95)
median = interpolate_nan_values(median) 
  
ax.plot(df0['time'].unique(), perc5, color='orange', linewidth=0.3)
ax.plot(df0['time'].unique(), perc25, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc75, color='orange', linewidth=0.5)
ax.plot(df0['time'].unique(), perc95, color='orange', linewidth=0.3)
    
ax.fill_between(df0['time'].unique(), perc75, perc25, alpha=0.3, color='orange', label='50% confidence interval')
ax.fill_between(df0['time'].unique(), perc95, perc5, alpha=0.3, color='orange', label='90% confidence interval')
ax.scatter(df0['time'].unique(), median, color='black', s=10)

# ax.legend(loc='upper center', fontsize = 'small')

dates = Q_daily.loc[(Q_daily['date'] > df0['time'].unique().min()) & (Q_daily['date'] < df0['time'].unique().max())]
# Create a twin Axes for the scatter plot
ax9 = ax.twinx()
# Plot the scatter plot on the right side axis
ax9.plot(Q_daily['date'], Q_daily['average_waarde'], color='grey')
# ax2.scatter(df0['time'], df0['Q5'], color='grey')
# Set the label and color for the right side axis
ax9.set_ylabel('discharge [m3/s]', color='grey')

def getGrowthFactor(arr):
    arr1 = arr[:-1]
    arr2 = arr[1:]
    garr = arr2/arr1
    return garr

name = hotspot + 'fileinfo.pkl'
file_info = pd.read_pickle(os.path.join(main_path, name))

file_info = file_info[file_info['TP Value'] == 0]
file_info = file_info.reset_index(drop=True)

dffname = hotspot + 'dff.pkl'
dff = pd.read_pickle(os.path.join(main_path, dffname))

timesteps = dff['time index'].unique()

median = np.full((len(timesteps),), np.nan)
dt = np.full((len(timesteps)-1,), np.nan)
for j, ts in enumerate(timesteps):
    if hotspot == 'H' and ts in [14,15,16,17,18]:
        continue
    if not j == len(timesteps)-1:
        curr = dff.loc[dff['time index'] == timesteps[j]]['time'].unique()[0]
        nxt = dff.loc[dff['time index'] == timesteps[j+1]]['time'].unique()[0]
        diff = (nxt - curr).astype('timedelta64[s]').item()
        dt[j] = float(diff.total_seconds()) / (60*60*24)
    data = dff.loc[dff['time index'] == ts]['height']
    if len(data) == 0:
        continue
    median[j] = data.median() 

median = interpolate_nan_values(median)
median = getGrowthFactor(median)**(1/(dt))
time = dff['time'].unique()
ax.plot(time[1:], median, color='red', label='Median', linestyle='--', linewidth=1)
ax.axhline(1, linewidth=0.3)


ax.set_ylabel('Daily Growth [1/day]')
ax.set_title('Swiping')
ax.set_ylim([0.9, 1.15])

ax1.text(.02, .99, 'a', ha='left', va='top', transform=ax1.transAxes)
ax2.text(.02, .99, 'd', ha='left', va='top', transform=ax2.transAxes)
ax3.text(.02, .99, 'b', ha='left', va='top', transform=ax3.transAxes)
ax4.text(.02, .99, 'e', ha='left', va='top', transform=ax4.transAxes)
ax5.text(.02, .99, 'c', ha='left', va='top', transform=ax5.transAxes)
ax6.text(.02, .99, 'f', ha='left', va='top', transform=ax6.transAxes)
ax7.text(.98, .99, 'g', ha='right', va='top', transform=ax7.transAxes)
ax8.text(.98, .99, 'h', ha='right', va='top', transform=ax8.transAxes)


# Set the tick labels
labels = ax8.get_xticklabels()
# Rotate the tick labels by 45 degrees
for label in labels:
    label.set_rotation(45)


ax1.tick_params(labelbottom=False)
ax2.tick_params(labelbottom=False, labelleft=False)
ax3.tick_params(labelbottom=False)
ax4.tick_params(labelbottom=False, labelleft=False)
ax6.tick_params(labelleft=False)
ax7.tick_params(labelbottom=False)

ax1.set_title('Topping')
ax2.set_title('Swiping')

ax5.legend(loc='lower left', fontsize=9)
ax6.legend(loc='lower left', fontsize=9)

# Set axis labels
ax5.set_xlabel('time [days]')
ax6.set_xlabel('time [days]')

ax1.set_ylabel('H/H0 [-]')
ax3.set_ylabel('L/L0 [-]')
ax5.set_ylabel('C/C0 [-]')


# Set axis limits
ax5.set_xlim([0, 50])
ax6.set_xlim([0, 50])


ax1.set_ylim([0, 2.5])
ax3.set_ylim([0, 2.5])
ax5.set_ylim([0, 2.5])


figname = 'HS-' + hotspot + '_Everything_new_data.png'
plt.tight_layout()
plt.savefig(os.path.join(out_path, 'DevOverTime', figname), dpi=300)

#%%








