# -*- coding: utf-8 -*-
"""
Created on Wed May 24 11:21:40 2023

@author: kruis
"""

#%% Settings

# Choose hotspot
hotspot = 'F'
# Set lower and upperbound for the Fourier wavelengths that should be included
lowerbound = 20
upperbound = 300
# Set width of the plough
plough_width = 6

#Plot Settings
# wdsmin, wdsmax, wxsmin, wxsmax = 800, 2800, 0, 280
wdsmin, wdsmax, wxsmin, wxsmax = 300, 2200, 50, 200          # HS F
elevation =2 

#%% Imports 

import numpy as np
import os
from datetime import datetime
import pandas as pd
import json
from scipy.signal import savgol_filter, argrelextrema
import math
import geopandas as gpd
import pickle
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable

import sys
sys.path.insert(1, 'C:/Users/kruis/Documents/Thesis/Python/Final')
import duneAnalysisFunctions as daf

import warnings
warnings.filterwarnings("ignore")

#%% Define paths
waveletOut_path = r'C:/Users/kruis/Documents/Thesis/Python/waveletOut'
hotspotData_path = r'C:/Users/kruis/Documents/Thesis/Python/hotspot_data.xlsx'
StrCo_path = r'C:/Users/kruis/Documents/Thesis/Python/StreamCoords'
fileInfo_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES/file_info.pkl'
testFigure_path = r'C:/Users/kruis/Documents/Thesis/Python/figures'
out_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut'
ploeg_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES'
hotspot_dredging_out_path = r'C:/Users/kruis/Documents/Thesis/Python/Final/dredging_dicts'
paper_path = r'C:/Users/kruis/Documents/Thesis/Overleaf_files'

#%% Initiate script

# Rework settings to be workable 
hs_string = 'ABCDEFGHIJ'
index = hs_string.find(hotspot)

# get transformed sn file
sn_name = 'HS_' + hotspot + '_sn.pkl'
sn = pd.read_pickle(os.path.join(StrCo_path,sn_name))


# Define minimum and maximum values of the Stream Coords grid
ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())
extent = [wdsmax - wdsmin, 0, wxsmin+xs_min, wxsmax+xs_min]

# Load the array with wavelengths that correspond to the indices 
wavelengths_file_name = 'HS_' + hotspot + '_wavelengths.npy'
wavelengths_all = np.load(os.path.join(waveletOut_path, hotspot, wavelengths_file_name))

# Dataframe with info about the hotspots
hotspot_data = pd.read_excel(hotspotData_path)
    
# Get the original MBES data in Stream Coords 
# Open dictionary for reading Zmax
Zmax_name = 'HS_' + hotspot + '_max.json'
with open(os.path.join(StrCo_path, Zmax_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
Zmax_dict = {}
for key, value in serialized_dict.items():
    Zmax_dict[datetime.fromisoformat(key)] = np.array(value)    
    
# Open dictionary for reading Zmin    
Zmin_name = 'HS_' + hotspot + '_min.json'
with open(os.path.join(StrCo_path, Zmin_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
Zmin_dict = {}
for key, value in serialized_dict.items():
    Zmin_dict[datetime.fromisoformat(key)] = np.array(value)        
 
# Load file info data about this hotspot that has inputs linking to the dictionary
file_info = pd.read_pickle(fileInfo_path)
file_info = file_info[file_info['Hotspot Letter'] == hotspot]
file_info = file_info[file_info.isin(Zmax_dict.keys()).any(axis=1)]
file_info.reset_index(inplace=True, drop=True)

# Create an array of Date Time objects for all time steps
datetime_array = file_info['Date Time'].values
 
# Load the average bed profile of the hotspot
Zavg_bed = np.load(os.path.join(waveletOut_path, hotspot, 'average_bed.npy')) 

# Load dredging and dumping dictionaries
name_dredge = 'HS_' + hotspot + '_dredging.pickle'
name_dump = 'HS_' + hotspot + '_dumping.pickle'
# Open a file for writing in binary mode
with open(os.path.join(hotspot_dredging_out_path, name_dredge), 'rb') as f:
    # Use the pickle.load() function to deserialize the dictionary from the file
    dredging_dict = pickle.load(f)
# Open a file for writing in binary mode
with open(os.path.join(hotspot_dredging_out_path, name_dump), 'rb') as f:
    # Use the pickle.load() function to deserialize the dictionary from the file
    dumping_dict = pickle.load(f)
    
# Determine the minimum and maximum x and y values in the hotspot from QGis
min_x, max_x = hotspot_data.loc[index, 'xmin'], hotspot_data.loc[index, 'xmax']
min_y, max_y = hotspot_data.loc[index, 'ymin'], hotspot_data.loc[index, 'ymax']

# Loads a dataframe with line coordinates for the plough (developed by script Pepijn and Andries)
lijnen = pd.read_pickle(os.path.join(ploeg_path,'lijnen.pkl'))
# Create a GeoDataFrame of the plough_data
lijnen = gpd.GeoDataFrame(lijnen)
lijnen = lijnen.cx[min_x:max_x, min_y:max_y]
lijnen = lijnen.reset_index()
# Transform plough lines to stream coords and turn them into polygons by adding the plough width
plough_strco = daf.plough_lines_to_polygons(lijnen, sn, plough_width, xs_min)

#%% Generate a 4D array containing 9 different bathymetries at each time interval

# Array with strings representing the different bathymetries
Znames = ['Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf']
# Determine all bathymetries for a chosen set of wavelengths
bathies = daf.generate_bathies(file_info.iloc[:], Znames, Zavg_bed, Zmax_dict, Zmin_dict, 
                           wavelengths_all, lowerbound, upperbound, 
                           waveletOut_path, hotspot)

#%%
# # Determine average displacement to next time step with Normalized Spatial Cross 
# Correlation on the Full Bathymetry
ds_shift_reg_array = np.full(80, np.nan)
for time in range(len(file_info)-1):
    time_diff = file_info.loc[time+1, 'Date Time'] - file_info.loc[time, 'Date Time']
    time_diff_weeks = time_diff.total_seconds() / (60*60*24*7)
    ds_shift_reg_array[time], ccf_reg = daf.nscc_full_bathymetry(bathies, time, time_diff_weeks)
    print(time)

#%% Determine time dependent data

# for time in range(len(file_info)-1):
time = 2
time_diff = file_info.loc[time+1, 'Date Time'] - file_info.loc[time, 'Date Time']
time_diff_weeks = time_diff.total_seconds() / (60*60*24*7)

# Determine average displacement to next time step with Normalized Spatial Cross 
# Correlation on the Full Bathymetry
# ds_shift_reg, ccf_reg = daf.nscc_full_bathymetry(bathies, time, time_diff_weeks)
ds_shift_reg = ds_shift_reg_array[time]

# Determine full crest locations
crests = daf.find_local_maximums(bathies, time, sn, xs_min, order=10, threshold_height=0)


# Determine analysis points
dunes = daf.determine_analysis_points(crests, xs_min=wxsmin, xs_max=wxsmax, ds_min=wdsmin, 
                                  ds_max=wdsmax, minimum_points=3, interval=20)

# Determine dune length in river axis direction
dunes = daf.get_dune_length_ds(bathies, time, dunes, upperbound, lowerbound, Zname='Zrec')

# Determine dredging quantities on a grid
dredging_grid = daf.calculate_quantity_grid(dredging_dict, file_info, time, Zavg_bed)

# Determine dumping quantities on a grid
dumping_grid = daf.calculate_quantity_grid(dumping_dict, file_info, time, Zavg_bed)

# Determine plough intensity
if time==0:
    plough_intensity = np.full((Zavg_bed.shape[0], Zavg_bed.shape[1]), np.nan)
else:
    plough_data = plough_strco.loc[plough_strco['date'].dt.date > file_info.loc[time-1, 'Date Time'].date()].loc[plough_strco['date'].dt.date <= file_info.loc[time, 'Date Time'].date()]
    plough_data.reset_index(drop=True)
    plough_data['id'] = range(1, len(plough_data)+1) # assign a unique ID to each polygon
    plough_data = plough_data.loc[:, ['poly', 'id']]
    plough_intensity = daf.rasterize_polygons(plough_data, Zavg_bed)
    plough_intensity = np.where(plough_intensity == 0, np.nan, plough_intensity)

#%% Normalized Spatial Cross Correlation - Individiual Analysis points - Variable window size

# Select version of the bathymetry to be used
# pick from these: 'Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf'
Zname = 'Zrec'
Zname_index = Znames.index(Zname)

# Set window sizes for NSCC.
ds_factor = 2           # 2 = window length equal to dune length in one dimensional direction
window_size_xs = 50     # 50 = window lenght of 100 m, covers a large part of the river width

# Perform NSCC on the analysis points
dunes = daf.nssc_individual_analysis_points(bathies, dunes, time, ds_shift_reg, 
                                            ds_factor, window_size_xs, Zname_index)

# Transform coordinates to regular XY values again
dunes = daf.convertToXY(dunes, sn, 'ds_new*', 'xs_new*' , 'X_new', 'Y_new')

# Only keep points with a correlation factor above a certain threshold value
dunes = dunes.loc[dunes['ccf']>=0.2]
dunes = dunes.reset_index(drop=True)
backup = dunes
print('average ccf = ' + str(backup['ccf'].mean()))

#%% Processing
dunes = backup
dunes = dunes.reset_index(drop=True)

# Set threshold height for determinining local max and minimum
threshold_height = -0.3

# Determine migration parameters per analysis point
dunes['migration SC'] = np.sqrt(dunes['xs_shift']**2 + dunes['ds_shift']**2)          
dunes['migration XY'] = np.sqrt((dunes['X_new']-(dunes['X']))**2 + (dunes['Y_new']-(dunes['Y']))**2) 
dunes['migration angle'] = np.degrees(np.arctan(dunes['xs_shift']/dunes['ds_shift']))
dunes['celerity SC'] = dunes['migration SC'] / time_diff_weeks
dunes['celerity XY'] = dunes['migration XY'] / time_diff_weeks

# Remove outliers, falling outside of the 2*standard deviation range 95% confidence interval for migration parameters
range_mig_dist, range_mig_angle = 2*dunes['migration SC'].std(), 2*dunes['migration angle'].std()
mean_mig_dist, mean_mig_angle = dunes['migration SC'].mean(), dunes['migration angle'].mean()
dunes = dunes[(dunes['migration SC'] >= mean_mig_dist - range_mig_dist) &
              (dunes['migration SC'] <= mean_mig_dist + range_mig_dist) &
              (dunes['migration angle'] >= mean_mig_angle - range_mig_angle) &
              (dunes['migration angle'] <= mean_mig_angle + range_mig_angle)]
dunes = dunes.reset_index(drop=True)

# Initiate empty arrays for determining dune parameters
length = np.full((len(dunes),), np.nan)
height = np.full((len(dunes),), np.nan)
lee_slope = np.full((len(dunes),), np.nan)
dredged_volume = np.full((len(dunes),), np.nan)
dredged_height = np.zeros(len(dunes))
dumped_volume = np.full((len(dunes),), np.nan)
ploughed_sum = np.full((len(dunes),), 0)
ploughed_max = np.full((len(dunes),), 0)
heightNAP = np.full((len(dunes),), np.nan)
dune_volume = np.full((len(dunes),), np.nan) 

for point_index in range(len(dunes)):
    
    if not point_index == 34:
        continue
    
    # Define the two points you want to connect
    point1 = (dunes.loc[point_index, 'ds'], dunes.loc[point_index, 'xs'])
    point2 = (dunes.loc[point_index, 'ds']+dunes.loc[point_index, 'ds_shift'], dunes.loc[point_index, 'xs']+dunes.loc[point_index, 'xs_shift'])
    
    # Calculate the distance and angle between the two points
    dds = point2[0] - point1[0]
    dxs = point2[1] - point1[1]
    distance = np.sqrt(dds**2 + dxs**2)
    angle = np.arctan2(dxs, dds)
    
    # Calculate the ds and xs coordinates of each point along the line
    ds_coords = np.linspace(point1[0] - upperbound/2*np.cos(angle), point1[0] + upperbound/2*np.cos(angle), num=301, endpoint=True)
    xs_coords = np.linspace(point1[1] - upperbound/2*np.sin(angle), point1[1] + upperbound/2*np.sin(angle), num=301, endpoint=True)
    
    try:
        # Reconstructed wavelet signal along the line
        elevation_rec = [bathies[time, 4, int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
        elevation_rec = np.array(elevation_rec)
        elevation_rec = savgol_filter(elevation_rec,7,3,mode='mirror',axis = 0)
        
        elevation_orig = [bathies[time, 3, int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
        elevation_orig = np.array(elevation_orig)
        elevation_orig = savgol_filter(elevation_orig,7,3,mode='mirror',axis = 0)
        
        dredging = [dredging_grid[int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
        dredging = np.array(dredging)
        # dredging = savgol_filter(dredging,7,3,mode='mirror',axis = 0)        

        dumping = [dumping_grid[int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
        dumping = np.array(dumping)
        # dumping = savgol_filter(dumping,7,3,mode='mirror',axis = 0) 
        
        ploughed = [plough_intensity[int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
        ploughed = np.array(ploughed)
        # ploughed = savgol_filter(ploughed,7,3,mode='mirror',axis = 0) 
        
        nonaveraged = [bathies[time, 5, int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
        nonaveraged = np.array(nonaveraged)
        # nonaveraged = savgol_filter(nonaveraged,7,3,mode='mirror',axis = 0)
        
    except IndexError:
        # This error is fine. This means that spatial cross correlation result is out of bounds.
        # Just skip this data.
        # print('transect out of bounds of bathymetry')                           
        continue
    
    try: 
        # Determine the coordinates of the crest and troughs on reconstructed profile
        maxima = argrelextrema(elevation_rec, np.greater, order=10)[0]
        maxima = maxima[elevation_rec[maxima] > threshold_height]
        
        max_xr = maxima[np.abs(maxima - upperbound/2).argmin()]
        max_yr = elevation_rec[max_xr]
        # Determine the coordinates of the troughs down- and upstream
        minima = argrelextrema(elevation_rec, np.less, order=10)[0]
        minima = minima[elevation_rec[minima] < -threshold_height]
        
        min_up_xr = minima[minima < upperbound/2].max()
        min_down_xr = minima[minima > upperbound/2].min()
        min_up_yr = elevation_rec[min_up_xr]
        min_down_yr = elevation_rec[min_down_xr]
        
        # Check if crest point is close to the analysis point
        if abs(max_xr - upperbound/2) > 20:                                        #!!! Check this value
            # print('closest crest too far from analysis point')
            continue
        
        # Determine the coordinates of the crest and troughs on the original profile
        max_xo = np.argmax(elevation_orig[min_up_xr:min_down_xr]) + min_up_xr
        max_yo = elevation_orig[max_xo]
        
        min_up_xo = np.nanargmin(elevation_orig[int(min_up_xr-lowerbound/2):int(min_up_xr+lowerbound/2)]) + int(min_up_xr-lowerbound/2)
        min_down_xo = np.nanargmin(elevation_orig[int(min_down_xr-lowerbound/2):int(min_down_xr+lowerbound/2)]) + int(min_down_xr-lowerbound/2)
        min_up_yo = elevation_orig[min_up_xo]
        min_down_yo = elevation_orig[min_down_xo]
        
        # Check if there is no elevation data missing between the troughs
        if np.isnan(elevation_orig[min_up_xr:min_down_xr]).any():
            # print('elevation profile between troughs is discontinuous')
            continue
        
        # Determine dune length and height
        length[point_index] = min_down_xo - min_up_xo
        height[point_index] = max_yo - (min_down_yo + min_up_yo)/2
        
        # Determine lee angle
        # defined as the mean slope of the middle 4/6th part of the dune lee slope (van der Mark et al., 2008)
        # echte horizontaal ivm zwaartekracht
        lee1_x, lee2_x = round(max_xo + (min_down_xo - max_xo)/6), round(max_xo + (min_down_xo - max_xo)*5/6)
        lee1_y, lee2_y = elevation_orig[lee1_x], elevation_orig[lee2_x]
        lee_slope[point_index] = np.arctan2((lee2_y-lee1_y),(lee2_x-lee1_x))
        
        # Determine dredged and dumped volume [in m3/m]
        dredged_volume[point_index] = np.nansum(dredging[min_up_xo:min_down_xo])
        H_dredged = np.nanmax(dredging[min_up_xo:min_down_xo])
        if not np.isnan(H_dredged):
            dredged_height[point_index] = H_dredged
        dumped_volume[point_index] = np.nansum(dumping[min_up_xo:min_down_xo])
        
        # Determine plough quantity
        ploughed_sum[point_index] = np.nansum(ploughed[min_up_xo:min_down_xo])
        max_plough = np.nanmax(ploughed[min_up_xo:min_down_xo])
        if not np.isnan(max_plough): 
            ploughed_max[point_index] = max_plough
        
        # Determine highest point w.r.t NAP
        heightNAP[point_index] = np.nanmax(nonaveraged[min_up_xo:min_down_xo])
        
        # Determine dune volume
        floor = np.linspace(min_up_yo,min_down_yo, min_down_xo-min_up_xo+1)
        dune_volume[point_index] = (elevation_orig[min_up_xo:min_down_xo+1] - floor).sum()
               
    except ValueError:
        # print('a value error occured')
        continue

dunes['length'] = length
dunes['height'] = height
dunes['aspect ratio'] = height / length
dunes['lee angle'] = np.degrees(lee_slope)

dunes['Zmax_NAP'] = heightNAP
dunes['dune volume'] = dune_volume

dunes['dredged volume'] = dredged_volume
dunes['dredged height'] = dredged_height
dunes['dumped volume'] = dumped_volume
dunes['ploughed_sum'] = ploughed_sum
dunes['ploughed_max'] = ploughed_max

dunes['time index'] = time
dunes['time'] = file_info.loc[time, 'Date Time']

# Drop rows containing NaN values
dunes.dropna(inplace=True)
# Filter make sure only points with a dune height of at least 0.1m are taken into account
dunes = dunes.loc[dunes['height']>=0.1]
# dunes = dunes.reset_index(drop=True)
print(str(len(dunes)) + ' valid analysis points remain')


#%% Plot Paper: Dune Shape 
import matplotlib.patches as patches
fig, ax = plt.subplots(2,1,figsize=(8,4), gridspec_kw={'height_ratios': [0.3, 0.7]})

ax[0].set_xlabel('ds* [m]')
ax[0].set_ylabel('xs* [m]')

im = ax[0].imshow(np.flip(bathies[time,4,wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='lower', vmin=-2, vmax=2)
ax[0].plot(ds_coords-wdsmin, xs_coords+xs_min, color='black')
ax[0].scatter(point1[0]-wdsmin, point1[1]+xs_min, color='Black', s=50, zorder=3, marker='x')

divider0 = make_axes_locatable(ax[0])
divider1 = make_axes_locatable(ax[1])

cax0 = divider0.append_axes('right', size='2%', pad=0.2)
cax1 = divider1.append_axes('right', size='2%', pad=0.2)
cbar = plt.colorbar(im, cax=cax0)
cbar.set_label('Z [m]', rotation=270)
cbar.ax.yaxis.labelpad = 10
cax1.set_xticks([])
cax1.set_yticks([])
cax1.set_frame_on(False)

# Plot the elevation profile
ax[1].plot(elevation_rec, label='Wavelet signal')
ax[1].plot(elevation_orig, label='Original signal')

ax[1].fill_between(np.linspace(min_up_xo,min_down_xo, len(floor)), elevation_orig[min_up_xo:min_down_xo+1], 
                    floor, alpha=0.3, color='orange', label='Area')
ax[1].fill_between(np.linspace(min_up_xo,min_down_xo, len(floor)), elevation_orig[min_up_xo:min_down_xo+1]+dredging[min_up_xo:min_down_xo+1], 
                    elevation_orig[min_up_xo:min_down_xo+1], alpha=0.3, color='purple', label='Area')

ax[1].plot(ploughed)
ax[1].plot(elevation_orig+dredging, color='purple', label='Before dredging')
ax[1].set_xlabel('Distance [m]')
ax[1].set_ylabel('Z (m)')
# Highlight point 1 in the plot
ax[1].scatter(max_xo, max_yo, color='red', s=50, zorder=3, label='Crest')
ax[1].scatter(min_down_xo, min_down_yo, color='green', s=50, zorder=3, label='Trough')
ax[1].scatter(min_up_xo, min_up_yo, color='green', s=50, zorder=3, label='_nolegend_')

ax[1].scatter(150, elevation_rec[150], color='black', s=50, zorder=3, marker='x', label='Analysis point')
ax[1].plot((min_up_xo, min_down_xo),(min_up_yo, min_down_yo), linewidth=0.5, linestyle='-', color='black', label='_nolegend_')

ax[1].plot((min_up_xo, min_up_xo),(min(min_up_yo, min_down_yo)-0.15, min_up_yo + 0.05), linewidth=0.5, linestyle='-', color='black', label='_nolegend_')
ax[1].plot((min_down_xo, min_down_xo),(min(min_up_yo, min_down_yo)-0.15, min_down_yo + 0.05), linewidth=0.5, linestyle='-', color='black', label='_nolegend_')
ax[1].plot((max_xo+5, (min_down_xo+min_up_xo)/2-5),(max_yo, max_yo), linewidth=0.5, linestyle='-', color='black', label='_nolegend_')


# Create the double-sided dashed arrow line
arrow_line = patches.FancyArrowPatch((min_down_xo, min(min_down_yo, min_up_yo) - 0.1), (min_up_xo,  min(min_down_yo, min_up_yo) - 0.1),
                                     arrowstyle='<|-|>', linestyle='dashed',
                                     color='green', mutation_scale=10)
ax[1].add_patch(arrow_line)
arrow_line = patches.FancyArrowPatch(((min_down_xo+min_up_xo)/2, (min_down_yo + min_up_yo)/2), ((min_down_xo+min_up_xo)/2,  max_yo),
                                     arrowstyle='<|-|>', linestyle='dashed',
                                     color='red', mutation_scale=10)
ax[1].add_patch(arrow_line)

ax[1].set_xlim((300,0))
ax[0].text(.01, .98, 'a', ha='left', va='top', transform=ax[0].transAxes)
ax[1].text(.01, .98, 'b', ha='left', va='top', transform=ax[1].transAxes)

legend = ax[1].legend(loc='lower right')
legend.get_frame().set_alpha(0.7)  # Set legend background transparency
legend.get_frame().set_edgecolor('gray')  # Set legend border color
legend.get_frame().set_linewidth(0.5)  # Set legend border width
plt.setp(legend.texts, fontsize='small')  # Set legend font size

plt.tight_layout()
plt.savefig(r'C:/Users/kruis/Documents/Thesis/Python/figures/dunedredge.png', dpi=300)
#%% Follow dredged area for next time steps



# # Determine search area (keep in mind, only xs is multiplied by 2)
# # if ds_shift_reg < 20:
# #     # if mean shift is very small, set some default values for the search area
# #     range_ds_min, range_ds_max = 0, 40
# #     range_xs = 5
# # else:
# #     # else set ds search area between 50% and 150% of the mean migration
# #     range_ds_min, range_ds_max = round(ds_shift_reg * 0.5), round(ds_shift_reg * 1.5)
# #     range_xs = round(ds_shift_reg/4)


# xs_shift = np.zeros(len(dunes))
# ds_shift = np.zeros(len(dunes))
# ccf = np.zeros(len(dunes))


# ds, xs = round(dredging_grid.shape[1]/2), round(dredging_grid.shape[0]/2)
# window_size_ds = ds
# window_size_xs = xs

# # Get window
# window0 = daf.getWindow(bathies[time, 4, :, :], xs, ds, window_size_xs, window_size_ds)
# window0 = window0[:-1, 1:-1]
# # Replace NaN values in window with dredging_grid values from array2
# nan_indices = np.isnan(dredging_grid)
# window0[nan_indices] = dredging_grid[nan_indices]

# time_ev = 5
# xs_shift = np.zeros(time_ev)
# ds_shift = np.zeros(time_ev)
# ccf = np.zeros(time_ev)

# for timestep in range(time_ev):
#     print(timestep)

#     if ds_shift_reg_array[time+timestep] < 20:
#         range_ds_min = round(0 + ds_shift_reg_array[time:time+timestep].sum())
#         range_ds_max = round(2*ds_shift_reg_array[time+timestep] + ds_shift_reg_array[time:time+timestep].sum())
#         range_xs = 10
#     else:
#         range_ds_min = round(ds_shift_reg_array[time+timestep] * 0.5 + ds_shift_reg_array[time:time+timestep].sum())
#         range_ds_max = round(ds_shift_reg_array[time+timestep] * 1.5 + ds_shift_reg_array[time:time+timestep].sum())
#         range_xs = 10
            
#     print(range_ds_min)
#     print(range_ds_max)   
#     ccc = np.zeros((range_xs * 2 + 1, range_ds_max - range_ds_min))
    
#     for dds in range(range_ds_max - range_ds_min):
#         print(str(dds) + '/' + str(range_ds_max - range_ds_min))
#         ds1 = ds + range_ds_min + dds
#         for dxs in range(range_xs * 2 + 1):
#             xs1 = xs - range_xs + dxs
    
#             window1 = daf.getWindow(bathies[time + timestep+1, Zname_index, :, :], xs1, ds1, window_size_xs, window_size_ds)
#             window1 = window1[:-1, 1:-1]
#             window1[nan_indices] = dredging_grid[nan_indices]
    
#             top = (window0 - np.nanmean(window0)) * (window1 - np.nanmean(window1))
#             bot1 = (window0 - np.nanmean(window0)) ** 2
#             bot2 = (window1 - np.nanmean(window1)) ** 2
#             ccc[dxs, dds] = np.nansum(top) / (np.nansum(bot1) * np.nansum(bot2)) ** 0.5
    
#     ccc = np.nan_to_num(ccc, nan=0)
#     xs_shift[timestep], ds_shift[timestep] = np.unravel_index(np.argmax(ccc), ccc.shape)
    
#     xs_shift[timestep] -= range_xs
#     ds_shift[timestep] += range_ds_min
#     ccf[timestep] = ccc.max()
    














#%% Plot Dredging
fig, ax = plt.subplots(figsize=(15,7))

im0 = ax.imshow(np.flip(bathies[time,4,wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation)
im1 = ax.imshow(np.flip(plough_intensity[wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='Oranges', origin='upper')
im2 = ax.imshow(np.flip(dredging_grid[wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='spring', origin='upper')
im3 = ax.imshow(np.flip(dumping_grid[wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='winter', origin='upper')
# Create the colorbar and adjust its position
cbar0 = fig.colorbar(im0, ax=ax, orientation='horizontal', aspect=50, pad=0.05)
cbar0.ax.set_xlabel('Elevation [m]')
cbar1 = fig.colorbar(im1, ax=ax, orientation='horizontal', aspect=50, pad=0.10)
cbar1.ax.set_xlabel('Number of plough movements [-]')
cbar2 = fig.colorbar(im2, ax=ax, orientation='horizontal', aspect=50, pad=0.15)
cbar2.ax.set_xlabel('Dredged volume (=depth on 1x1 grid) [m3]')
cbar3 = fig.colorbar(im3, ax=ax, orientation='horizontal', aspect=50, pad=0.20)
cbar3.ax.set_xlabel('Dumped volume (=height on 1x1 grid) [m3]')
ax.set_xlabel('ds* [m]')
ax.set_ylabel('xs* [m]')
scatter = crests.loc[((crests['ds'] >= wdsmin) & (crests['ds'] <= wdsmax)) & ((crests['xs'] >= wxsmin) & (crests['xs'] <= wxsmax))]
ax.scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')      #!!! Check -1
plt.title('Dredging locations HS-F, time: ' + str(str(file_info.loc[time, 'Date Time'])), fontsize=16, fontweight='bold')
plt.tight_layout()
plt.savefig(os.path.join(out_path,'test','pres','dredging_locations.png'), dpi = 300)



#%% Plot Paper: Wavelet


fig, ax = plt.subplots(3,1,figsize=(8,6), gridspec_kw={'height_ratios': [0.4, 0.3, 0.3]})

xs = -xs_min        # Plots rive axis

ax[0].plot(bathies[time,3,xs,wdsmin:wdsmax], linewidth=0.5, color='black', label='Original signal')
ax[0].plot(bathies[time,4,xs,wdsmin:wdsmax], color='red', label='Wavelet signal')
ax[0].invert_xaxis()
ax[0].set_xlabel('ds [m]')
ax[0].set_ylabel('z [m]')
ax[0].set_title('1D signals along river axis')
ax[0].set_ylim([-elevation, elevation])
ax[0].set_xlim([wdsmax - wdsmin, 0])
ax[0].legend(ncol=2, loc='upper center')


im1 = ax[1].imshow(np.flip(bathies[time,3,wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='terrain', origin='upper', vmin=-elevation, vmax=elevation) # set vmin and vmax to the minimum and maximum values in the array
im2 = ax[2].imshow(np.flip(bathies[time,4,wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation)

# # Create the colorbar and adjust its position
# cbar = fig.colorbar(im, ax=ax[1], orientation='horizontal', aspect=50, pad=0.3)
# cbar.ax.set_xlabel('Z [m]')
ax[1].axhline(0, color='red', linestyle='--', linewidth=0.7)
ax[1].set_xlabel('ds [m*]')
ax[1].set_ylabel('xs [m]')
# ax[1].grid(axis='x', color='k', linestyle='-', linewidth=0.5, which='both')
# ax[1].xaxis.set_minor_locator(ticker.AutoMinorLocator())
ax[1].set_title('Original bed elevation')

ax[2].axhline(0, color='red', linestyle='--', linewidth=0.7)
ax[2].set_xlabel('ds [m*]')
ax[2].set_ylabel('xs [m]')
# ax[2].grid(axis='x', color='k', linestyle='-', linewidth=0.5, which='both')
# ax[2].xaxis.set_minor_locator(ticker.AutoMinorLocator())
ax[2].set_title('Wavelet reconstruction for wavelengths 20-300m')
scatter = crests.loc[((crests['ds'] >= wdsmin) & (crests['ds'] <= wdsmax)) & ((crests['xs'] >= wxsmin) & (crests['xs'] <= wxsmax))]
# ax[1].scatter(scatter['ds']-1100, scatter['xs*'], s=0.1, alpha=1, c='black')
ax[2].scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')      #!!! Check -1

divider0 = make_axes_locatable(ax[0])
divider1 = make_axes_locatable(ax[1])
divider2 = make_axes_locatable(ax[2])
cax0 = divider0.append_axes('right', size='2%', pad=0.2)
cax1 = divider1.append_axes('right', size='2%', pad=0.2)
cax2 = divider2.append_axes('right', size='2%', pad=0.2)
cbar1 = plt.colorbar(im1, cax=cax1)
cbar1.set_label('Z [m+NAP]', rotation=270)
cbar1.ax.yaxis.labelpad = 10
cbar2 = plt.colorbar(im2, cax=cax2)
cbar2.set_label('Z [m]', rotation=270)
cbar2.ax.yaxis.labelpad = 10

# cax0.grid(False)
cax0.set_xticks([])
cax0.set_yticks([])
cax0.set_frame_on(False)

# Adjust the position of the middle subplot
fig.subplots_adjust(hspace=0.7)
# # Add a main title to the plot
fig.suptitle('Hotspot-'+hotspot+': '+str(file_info.loc[time, 'Date Time'].date()))

ax[0].text(.01, .98, 'a', ha='left', va='top', transform=ax[0].transAxes)
ax[1].text(.01, .98, 'b', ha='left', va='top', transform=ax[1].transAxes)
ax[2].text(.01, .98, 'c', ha='left', va='top', transform=ax[2].transAxes)

plt.tight_layout()


#%% Plot Paper: NSCC
dunes = backup


dunes = dunes.loc[((dunes['ds'] >= wdsmin) & (dunes['ds'] <= wdsmax)) & ((dunes['xs'] >= wxsmin) & (dunes['xs'] <= wxsmax))]
dunes = dunes.loc[(dunes['ds']+dunes['ds_shift'] <= wdsmax) & (dunes['xs*']+dunes['xs_shift'] < wxsmax+xs_min) & (dunes['xs*']+dunes['xs_shift'] > wxsmin+xs_min)]
scatter = crests.loc[((crests['ds'] >= wdsmin) & (crests['ds'] <= wdsmax)) & ((crests['xs'] >= wxsmin) & (crests['xs'] <= wxsmax))]


# Select version of the bathymetry to be used
plot_map = 'Zrec'     # pick from these: ['Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf']

fig, ax = plt.subplots(2,1, figsize=(10,5))
im = ax[0].imshow(np.flip(bathies[time,Znames.index(plot_map),wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation) # set vmin and vmax to the minimum and maximum values in the array


# ax.scatter(troughs['ds'], troughs['xs'], s=0.01, c='black')
# ax[0].scatter(crests['ds']-wdsmin, crests['xs*'], s=0.01, alpha=0.5, c='black')
ax[0].scatter(dunes['ds']-wdsmin, dunes['xs*']*-1, s=3, c='black')
ax[0].scatter(dunes['ds']+dunes['ds_shift']-wdsmin, (dunes['xs*']+dunes['xs_shift'])*-1, s=3, c='green')
ax[0].plot([dunes['ds']-wdsmin,dunes['ds']+dunes['ds_shift']-wdsmin],[dunes['xs*']*-1,(dunes['xs*']+dunes['xs_shift'])*-1], linewidth=0.3, c='green')

ax[0].set_xlabel('ds* [m]')
ax[0].set_ylabel('xs* [m]')

time0 = file_info.loc[time, 'Date Time']
ax[0].set_title(time0)

im = ax[1].imshow(np.flip(bathies[time+1,Znames.index(plot_map),wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation) # set vmin and vmax to the minimum and maximum values in the array

# Create the colorbar and adjust its position

divider0 = make_axes_locatable(ax[0])
cax0 = divider0.append_axes('top', size='10%', pad=0.5)
cax0.set_xticks([])
cax0.set_yticks([])
cax0.set_frame_on(False)

divider = make_axes_locatable(ax[1])
cax = divider.append_axes('bottom', size='10%', pad=0.5)
cbar = fig.colorbar(im, cax=cax, orientation='horizontal')
cbar.set_label('Z [m]')
cbar.ax.xaxis.labelpad = 10

ax[1].scatter(dunes['ds']-wdsmin, dunes['xs*']*-1, s=3, c='black')
ax[1].scatter(dunes['ds']+dunes['ds_shift']-wdsmin, (dunes['xs*']+dunes['xs_shift'])*-1, s=3, c='green')
ax[1].plot([dunes['ds']-wdsmin,dunes['ds']+dunes['ds_shift']-wdsmin],[dunes['xs*']*-1,(dunes['xs*']+dunes['xs_shift'])*-1], linewidth=0.3, c='green')

ax[1].set_xlabel('ds* [m]')
ax[1].set_ylabel('xs* [m]')

time1 = file_info.loc[time+1, 'Date Time']
ax[1].set_title(time1)

ax[0].scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')
ax[1].scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')

ax[0].text(.01, .98, 'a', ha='left', va='top', transform=ax[0].transAxes)
ax[1].text(.01, .98, 'b', ha='left', va='top', transform=ax[1].transAxes)

td = time1 - time0
fig.suptitle('HS-F: Spatial Cross Correlation - Time Interval: ' + str(td), fontsize=14)
print('Correlation map: ' + Zname)
print('Plot map: ' + plot_map)
plt.tight_layout()
# plt.savefig(os.path.join(paper_path,'nscc.png'), dpi = 300)











































