# -*- coding: utf-8 -*-
"""
Created on Sun May 28 13:06:31 2023

@author: kruis
"""

#%% Settings

# Choose hotspot
hotspot = 'F'
# Set lower and upperbound for the Fourier wavelengths that should be included
lowerbound = 20
upperbound = 300
# Set width of the plough
plough_width = 6

#Plot Settings
# wdsmin, wdsmax, wxsmin, wxsmax = 150, 2200, 60, 210          # HS E
# wdsmin, wdsmax, wxsmin, wxsmax = 500, 1400, 65, 190         # HS J
# wdsmin, wdsmax, wxsmin, wxsmax = 800, 2800, 0, 280        # HS H
wdsmin, wdsmax, wxsmin, wxsmax = 300, 2200, 50, 200          # HS F
elevation = 2

#%% Imports 

import numpy as np
import os
from datetime import datetime
import pandas as pd
import json
from scipy.signal import argrelextrema
import math
import geopandas as gpd
import pickle

import sys
sys.path.insert(1, 'C:/Users/kruis/Documents/Thesis/Python/Final')
import duneAnalysisFunctions as daf

import warnings
warnings.filterwarnings("ignore")


#%% Define paths
waveletOut_path = r'C:/Users/kruis/Documents/Thesis/Python/waveletOut'
hotspotData_path = r'C:/Users/kruis/Documents/Thesis/Python/hotspot_data.xlsx'
StrCo_path = r'C:/Users/kruis/Documents/Thesis/Python/StreamCoords'
fileInfo_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES/file_info.pkl'
testFigure_path = r'C:/Users/kruis/Documents/Thesis/Python/figures'
out_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut'
ploeg_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES'
hotspot_dredging_out_path = r'C:/Users/kruis/Documents/Thesis/Python/Final/dredging_dicts'
paper_path = r'C:/Users/kruis/Documents/Thesis/Overleaf_files'

#%% Initiate script

# Rework settings to be workable 
hs_string = 'ABCDEFGHIJ'
index = hs_string.find(hotspot)

# get transformed sn file
sn_name = 'HS_' + hotspot + '_sn.pkl'
sn = pd.read_pickle(os.path.join(StrCo_path,sn_name))

# Define minimum and maximum values of the Stream Coords grid
ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())

# Load the array with wavelengths that correspond to the indices 
wavelengths_file_name = 'HS_' + hotspot + '_wavelengths.npy'
wavelengths_all = np.load(os.path.join(waveletOut_path, hotspot, wavelengths_file_name))

# Dataframe with info about the hotspots
hotspot_data = pd.read_excel(hotspotData_path)
    
# Get the original MBES data in Stream Coords 
# Open dictionary for reading Zmax
Zmax_name = 'HS_' + hotspot + '_max.json'
with open(os.path.join(StrCo_path, Zmax_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
Zmax_dict = {}
for key, value in serialized_dict.items():
    Zmax_dict[datetime.fromisoformat(key)] = np.array(value)    
    
# Open dictionary for reading Zmin    
Zmin_name = 'HS_' + hotspot + '_min.json'
with open(os.path.join(StrCo_path, Zmin_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
Zmin_dict = {}
for key, value in serialized_dict.items():
    Zmin_dict[datetime.fromisoformat(key)] = np.array(value)        
 
# Load file info data about this hotspot that has inputs linking to the dictionary
file_info = pd.read_pickle(fileInfo_path)
file_info = file_info[file_info['Hotspot Letter'] == hotspot]
file_info = file_info[file_info.isin(Zmax_dict.keys()).any(axis=1)]
file_info.reset_index(inplace=True, drop=True)

# Create an array of Date Time objects for all time steps
datetime_array = file_info['Date Time'].values
 
# Load the average bed profile of the hotspot
Zavg_bed = np.load(os.path.join(waveletOut_path, hotspot, 'average_bed.npy')) 

# Load dredging and dumping dictionaries
name_dredge = 'HS_' + hotspot + '_dredging.pickle'
name_dump = 'HS_' + hotspot + '_dumping.pickle'
# Open a file for writing in binary mode
with open(os.path.join(hotspot_dredging_out_path, name_dredge), 'rb') as f:
    # Use the pickle.load() function to deserialize the dictionary from the file
    dredging_dict = pickle.load(f)
# Open a file for writing in binary mode
with open(os.path.join(hotspot_dredging_out_path, name_dump), 'rb') as f:
    # Use the pickle.load() function to deserialize the dictionary from the file
    dumping_dict = pickle.load(f)
    
# Determine the minimum and maximum x and y values in the hotspot from QGis
min_x, max_x = hotspot_data.loc[index, 'xmin'], hotspot_data.loc[index, 'xmax']
min_y, max_y = hotspot_data.loc[index, 'ymin'], hotspot_data.loc[index, 'ymax']

# Loads a dataframe with line coordinates for the plough (developed by script Pepijn and Andries)
lijnen = pd.read_pickle(os.path.join(ploeg_path,'lijnen.pkl'))
# Create a GeoDataFrame of the plough_data
lijnen = gpd.GeoDataFrame(lijnen)
lijnen = lijnen.cx[min_x:max_x, min_y:max_y]
lijnen = lijnen.reset_index()
# Transform plough lines to stream coords and turn them into polygons by adding the plough width
plough_strco = daf.plough_lines_to_polygons(lijnen, sn, plough_width, xs_min)

#%% Generate a 4D array containing 9 different bathymetries at each time interval

# Array with strings representing the different bathymetries
Znames = ['Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf']
# Determine all bathymetries for a chosen set of wavelengths
bathies0 = daf.generate_bathies(file_info.iloc[:], Znames, Zavg_bed, Zmax_dict, Zmin_dict, 
                           wavelengths_all, lowerbound, upperbound, 
                           waveletOut_path, hotspot)

if hotspot == 'E':
    file_info.loc[28, 'TP Value'] =0
    
file_info = file_info.loc[file_info['TP Value']==0]
UP_indices = np.array(file_info.index)
file_info['Tindex*'] = UP_indices
bathies = bathies0[UP_indices,:,:,:]
file_info = file_info.reset_index(drop=True)

#%%

# # Determine average displacement to next time step with Normalized Spatial Cross 
# Correlation on the Full Bathymetry
ds_shift_reg_array = np.full(len(file_info), np.nan)
for time in range(len(file_info)-1):
    time_diff = file_info.loc[time+1, 'Date Time'] - file_info.loc[time, 'Date Time']
    time_diff_weeks = time_diff.total_seconds() / (60*60*24*7)
    ds_shift_reg_array[time], ccf_reg = daf.nscc_full_bathymetry(bathies, time, time_diff_weeks)
    print(str(time)+'/'+str(len(file_info)))
file_info['ds_shift'] = ds_shift_reg_array
file_info_name = 'HS_'+hotspot+'_FI.pkl'
file_info.to_pickle(os.path.join(out_path,file_info_name))

#%% Create mother dataframe for all analysis points at all timesteps
all_analysis_points = pd.DataFrame()

file_info_name = 'HS_'+hotspot+'_FI.pkl'
file_info = pd.read_pickle(os.path.join(out_path,file_info_name))                                ### Check time step
ds_shift_reg_array = np.array(file_info['ds_shift'])

#%% Determine time dependent data
file_info = file_info.reset_index(drop=True)
for time in range(len(file_info)-1):

    print('current time index = ' + str(time))
    print('   - Getting time specific data')
    time_diff = file_info.loc[time+1, 'Date Time'] - file_info.loc[time, 'Date Time']
    time_diff_weeks = time_diff.total_seconds() / (60*60*24*7)
    
    if not time == 0:
        time_diff_prev = file_info.loc[time, 'Date Time'] - file_info.loc[time-1, 'Date Time']
        time_diff_prev_hours = time_diff_prev.total_seconds() / (60*60)
    else:
        time_diff_prev_hours = 999999
      
    # Determine average displacement to next time step with Normalized Spatial Cross 
    # Correlation on the Full Bathymetry
    # ds_shift_reg, ccf_reg = daf.nscc_full_bathymetry(bathies, time, time_diff_weeks)
    ds_shift_reg = ds_shift_reg_array[time]
    
    # Determine full crest locations
    crests = daf.find_local_maximums(bathies, time, sn, order=10, threshold_height=0)
    
    # Determine analysis points
    dunes = daf.determine_analysis_points(crests, xs_min=wxsmin, xs_max=wxsmax, ds_min=wdsmin, 
                                      ds_max=wdsmax, minimum_points=3, interval=10)
    
    # Determine dune length in river axis direction
    dunes = daf.get_dune_length_ds(bathies, time, dunes, upperbound, lowerbound, Zname='Zrec')
    
    # To counter TP values messing up the dredging data
    if time_diff_prev_hours > 12:
    
        # Determine dredging quantities on a grid
        dredging_grid = daf.calculate_quantity_grid(dredging_dict, file_info, time, Zavg_bed)
        
        # Determine dumping quantities on a grid
        dumping_grid = daf.calculate_quantity_grid(dumping_dict, file_info, time, Zavg_bed)
        
        # Determine plough intensity
        if time==0:
            plough_intensity = np.full((Zavg_bed.shape[0], Zavg_bed.shape[1]), np.nan)
        else:
            plough_data = plough_strco.loc[plough_strco['date'].dt.date > file_info.loc[time-1, 'Date Time'].date()].loc[plough_strco['date'].dt.date <= file_info.loc[time, 'Date Time'].date()]
            plough_data.reset_index(drop=True)
            plough_data['id'] = range(1, len(plough_data)+1) # assign a unique ID to each polygon
            plough_data = plough_data.loc[:, ['poly', 'id']]
            plough_intensity = daf.rasterize_polygons(plough_data, Zavg_bed)
            plough_intensity = np.where(plough_intensity == 0, np.nan, plough_intensity)
    
    # Normalized Spatial Cross Correlation - Individiual Analysis points - Variable window size
    print('   - Performing NSCC for ' + str(len(dunes)) + ' analysis points')
    # Select version of the bathymetry to be used
    # pick from these: 'Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf'
    Zname = 'Zrec'
    Zname_index = Znames.index(Zname)
    
    # Set window sizes for NSCC.
    ds_factor = 2           # 2 = window length equal to dune length in one dimensional direction
    window_size_xs = 50     # 50 = window lenght of 100 m, covers a large part of the river width
    
    # Perform NSCC on the analysis points
    dunes = daf.nssc_individual_analysis_points(bathies, dunes, time, ds_shift_reg, 
                                                ds_factor, window_size_xs, Zname_index)
     
    # Transform coordinates to regular XY values again
    dunes = daf.convertToXY(dunes, sn, 'ds_new*', 'xs_new*' , 'X_new', 'Y_new')
    
    # Only keep points with a correlation factor above a certain threshold value
    dunes = dunes.loc[dunes['ccf']>=0.2]
    dunes = dunes.reset_index(drop=True)
    
    # Determine migration parameters per analysis point
    dunes['migration SC'] = np.sqrt(dunes['xs_shift']**2 + dunes['ds_shift']**2)          
    dunes['migration XY'] = np.sqrt((dunes['X_new']-(dunes['X']))**2 + (dunes['Y_new']-(dunes['Y']))**2) 
    dunes['migration angle'] = np.degrees(np.arctan(dunes['xs_shift']/dunes['ds_shift']))
    dunes['celerity SC'] = dunes['migration SC'] / time_diff_weeks
    dunes['celerity XY'] = dunes['migration XY'] / time_diff_weeks
    
    # Remove outliers, falling outside of the 2*standard deviation range 95% confidence interval for migration parameters
    range_mig_dist, range_mig_angle = 2*dunes['migration SC'].std(), 2*dunes['migration angle'].std()
    mean_mig_dist, mean_mig_angle = dunes['migration SC'].mean(), dunes['migration angle'].mean()
    dunes = dunes[(dunes['migration SC'] >= mean_mig_dist - range_mig_dist) &
                  (dunes['migration SC'] <= mean_mig_dist + range_mig_dist) &
                  (dunes['migration angle'] >= mean_mig_angle - range_mig_angle) &
                  (dunes['migration angle'] <= mean_mig_angle + range_mig_angle)]
    dunes = dunes.reset_index(drop=True)

    print('   - Determining shape parameters for remaining ' + str(len(dunes)) + ' analysis points')    

    # Set threshold height for determinining local max and minimum
    threshold_height = -0.3
    
    # Initiate empty arrays for determining dune parameters
    length = np.full((len(dunes),), np.nan)
    height = np.full((len(dunes),), np.nan)
    lee_slope = np.full((len(dunes),), np.nan)
    dredged_volume = np.full((len(dunes),), np.nan)
    dredged_height = np.zeros(len(dunes))
    dumped_volume = np.full((len(dunes),), np.nan)
    ploughed_sum = np.full((len(dunes),), 0)
    ploughed_max = np.full((len(dunes),), 0)
    heightNAP = np.full((len(dunes),), np.nan)
    dune_volume = np.full((len(dunes),), np.nan)
    
    for point_index in range(len(dunes)):
        
        # Define the two points you want to connect
        point1 = (dunes.loc[point_index, 'ds'], dunes.loc[point_index, 'xs'])
        point2 = (dunes.loc[point_index, 'ds']+dunes.loc[point_index, 'ds_shift'], dunes.loc[point_index, 'xs']+dunes.loc[point_index, 'xs_shift'])
        
        # Calculate the distance and angle between the two points
        dds = point2[0] - point1[0]
        dxs = point2[1] - point1[1]
        distance = np.sqrt(dds**2 + dxs**2)
        angle = np.arctan2(dxs, dds)
        
        # Calculate the ds and xs coordinates of each point along the line
        ds_coords = np.linspace(point1[0] - upperbound/2*np.cos(angle), point1[0] + upperbound/2*np.cos(angle), num=301, endpoint=True)
        xs_coords = np.linspace(point1[1] - upperbound/2*np.sin(angle), point1[1] + upperbound/2*np.sin(angle), num=301, endpoint=True)
        
        try:
            # Reconstructed wavelet signal along the line
            elevation_rec = [bathies[time, 4, int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
            elevation_rec = np.array(elevation_rec)
            # elevation_rec = savgol_filter(elevation_rec,7,3,mode='mirror',axis = 0)
            
            elevation_orig = [bathies[time, 3, int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
            elevation_orig = np.array(elevation_orig)
            # elevation_orig = savgol_filter(elevation_orig,7,3,mode='mirror',axis = 0)
            
            dredging = [dredging_grid[int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
            dredging = np.array(dredging)
            # dredging = savgol_filter(dredging,7,3,mode='mirror',axis = 0)        
    
            dumping = [dumping_grid[int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
            dumping = np.array(dumping)
            # dumping = savgol_filter(dumping,7,3,mode='mirror',axis = 0) 
            
            ploughed = [plough_intensity[int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
            ploughed = np.array(ploughed)
            # ploughed = savgol_filter(ploughed,7,3,mode='mirror',axis = 0) 
            
            nonaveraged = [bathies[time, 5, int(round(xs)), int(round(ds))] for ds, xs in zip(ds_coords, xs_coords)]
            nonaveraged = np.array(nonaveraged)
            # nonaveraged = savgol_filter(nonaveraged,7,3,mode='mirror',axis = 0)
            
        except IndexError:
            # This error is fine. This means that spatial cross correlation result is out of bounds.
            # Just skip this data.
            # print('transect out of bounds of bathymetry')                           
            continue
        
        try: 
            # Determine the coordinates of the crest and troughs on reconstructed profile
            maxima = argrelextrema(elevation_rec, np.greater, order=10)[0]
            maxima = maxima[elevation_rec[maxima] > threshold_height]
            
            max_xr = maxima[np.abs(maxima - upperbound/2).argmin()]
            max_yr = elevation_rec[max_xr]
            # Determine the coordinates of the troughs down- and upstream
            minima = argrelextrema(elevation_rec, np.less, order=10)[0]
            minima = minima[elevation_rec[minima] < -threshold_height]
            
            min_up_xr = minima[minima < upperbound/2].max()
            min_down_xr = minima[minima > upperbound/2].min()
            min_up_yr = elevation_rec[min_up_xr]
            min_down_yr = elevation_rec[min_down_xr]
            
            # Check if crest point is close to the analysis point
            if abs(max_xr - upperbound/2) > 40:                                        
                # print('closest crest too far from analysis point')
                continue
            
            # Determine the coordinates of the crest and troughs on the original profile
            max_xo = np.argmax(elevation_orig[min_up_xr:min_down_xr]) + min_up_xr
            max_yo = elevation_orig[max_xo]
            
            min_up_xo = np.nanargmin(elevation_orig[int(min_up_xr-lowerbound/2):int(min_up_xr+lowerbound/2)]) + int(min_up_xr-lowerbound/2)
            min_down_xo = np.nanargmin(elevation_orig[int(min_down_xr-lowerbound/2):int(min_down_xr+lowerbound/2)]) + int(min_down_xr-lowerbound/2)
            min_up_yo = elevation_orig[min_up_xo]
            min_down_yo = elevation_orig[min_down_xo]
            
            # Check if there is no elevation data missing between the troughs
            if np.isnan(elevation_orig[min_up_xr:min_down_xr]).any():
                # print('elevation profile between troughs is discontinuous')
                continue
            
            # Determine dune length and height
            length[point_index] = min_down_xo - min_up_xo
            height[point_index] = max_yo - (min_down_yo + min_up_yo)/2
            
            # Determine lee angle
            # defined as the mean slope of the middle 4/6th part of the dune lee slope (van der Mark et al., 2008)
            # echte horizontaal ivm zwaartekracht
            lee1_x, lee2_x = round(max_xo + (min_down_xo - max_xo)/6), round(max_xo + (min_down_xo - max_xo)*5/6)
            lee1_y, lee2_y = elevation_orig[lee1_x], elevation_orig[lee2_x]
            lee_slope[point_index] = np.arctan2((lee2_y-lee1_y),(lee2_x-lee1_x))
            
            # Determine dredged and dumped volume [in m3/m]
            dredged_volume[point_index] = np.nansum(dredging[min_up_xo:min_down_xo])
            H_dredged = np.nanmax(dredging[min_up_xo:min_down_xo])
            if not np.isnan(H_dredged): 
                dredged_height[point_index] = H_dredged
            dumped_volume[point_index] = np.nansum(dumping[min_up_xo:min_down_xo])
            
            # Determine plough quantity
            ploughed_sum[point_index] = np.nansum(ploughed[min_up_xo:min_down_xo])
            max_plough = np.nanmax(ploughed[min_up_xo:min_down_xo])
            if not np.isnan(max_plough): 
                ploughed_max[point_index] = max_plough
            
            # Determine highest point w.r.t NAP
            heightNAP[point_index] = np.nanmax(nonaveraged[min_up_xo:min_down_xo])
            
            # Determine dune volume
            floor = np.linspace(min_up_yo,min_down_yo, min_down_xo-min_up_xo+1)
            dune_volume[point_index] = (elevation_orig[min_up_xo:min_down_xo+1] - floor).sum()
            
        except ValueError:
            # print('a value error occured')
            continue
    
    dunes['length'] = length
    dunes['height'] = height
    dunes['aspect ratio'] = height / length
    dunes['lee angle'] = abs(np.degrees(lee_slope))
    
    dunes['Zmax_NAP'] = heightNAP
    dunes['dune volume'] = dune_volume
    
    dunes['dredged volume'] = dredged_volume
    dunes['dredged height'] = dredged_height
    dunes['dumped volume'] = dumped_volume
    dunes['ploughed_sum'] = ploughed_sum
    dunes['ploughed_max'] = ploughed_max
    
    dunes['time index'] = time
    dunes['time'] = file_info.loc[time, 'Date Time']
    
    # Drop rows containing NaN values
    dunes.dropna(inplace=True)
    # Filter make sure only points with a dune height of at least 0.1m are taken into account
    dunes = dunes.loc[dunes['height']>=0.1]
    dunes = dunes.reset_index(drop=True)
    print('   - ' + str(len(dunes)) + ' analysis points remain')

    all_analysis_points = all_analysis_points.append(dunes, ignore_index=True)

# Store the resulting dataframe in a pickle file
dataframe_name = 'HS_' + hotspot + '_all_data_rerun.pkl'
all_analysis_points.to_pickle(os.path.join(out_path,dataframe_name))





#%%
import matplotlib.pyplot as plt
bathies = bathies0.copy()

time = 33

wdsmin, wdsmax, wxsmin, wxsmax = 150, 2200, 60, 210          # HS F

extent = [wdsmax - wdsmin, 0, wxsmin+xs_min, wxsmax+xs_min]

fig, ax = plt.subplots(figsize=(15,7))

im0 = ax.imshow(np.flip(bathies[time,4,wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation)





