# -*- coding: utf-8 -*-
"""
Created on Fri Jun 16 10:35:18 2023

@author: kruis
"""

#%% Settings

# Choose hotspot
hotspot = 'H'
# Set lower and upperbound for the Fourier wavelengths that should be included
lowerbound = 20
upperbound = 300
# Set width of the plough
plough_width = 6

#Plot Settings
# wdsmin, wdsmax, wxsmin, wxsmax = 150, 2200, 60, 210          # HS E
# wdsmin, wdsmax, wxsmin, wxsmax = 500, 1400, 65, 140        # HS J
wdsmin, wdsmax, wxsmin, wxsmax = 800, 2800, 0, 280        # HS H
# wdsmin, wdsmax, wxsmin, wxsmax = 300, 2200, 50, 200          # HS F
elevation = 2

#%% Imports 

import numpy as np
import os
from datetime import datetime
import pandas as pd
import json
from scipy.signal import savgol_filter, argrelextrema
import math
import geopandas as gpd
import pickle
import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable

import sys
sys.path.insert(1, 'C:/Users/kruis/Documents/Thesis/Python/Final')
import duneAnalysisFunctions as daf

import warnings
warnings.filterwarnings("ignore")


#%% Define paths
waveletOut_path = r'C:/Users/kruis/Documents/Thesis/Python/waveletOut'
hotspotData_path = r'C:/Users/kruis/Documents/Thesis/Python/hotspot_data.xlsx'
StrCo_path = r'C:/Users/kruis/Documents/Thesis/Python/StreamCoords'
fileInfo_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES/file_info.pkl'
testFigure_path = r'C:/Users/kruis/Documents/Thesis/Python/figures'
out_path = r'C:/Users/kruis/Documents/Thesis/Python/2dDuneAnalysisOut'
ploeg_path = r'C:/Users/kruis/Documents/Thesis/Python/processedMBES'
hotspot_dredging_out_path = r'C:/Users/kruis/Documents/Thesis/Python/Final/dredging_dicts'
paper_path = r'C:/Users/kruis/Documents/Thesis/Overleaf_files'

#%% Initiate script

# Rework settings to be workable 
hs_string = 'ABCDEFGHIJ'
index = hs_string.find(hotspot)

# get transformed sn file
sn_name = 'HS_' + hotspot + '_sn.pkl'
sn = pd.read_pickle(os.path.join(StrCo_path,sn_name))

# Define minimum and maximum values of the Stream Coords grid
ds_min, ds_max = math.floor(sn['ds'].min()), math.ceil(sn['ds'].max())
xs_min, xs_max = math.floor(sn['xs'].min()), math.ceil(sn['xs'].max())
extent = [wdsmax - wdsmin, 0, wxsmin+xs_min, wxsmax+xs_min]

# Load the array with wavelengths that correspond to the indices 
wavelengths_file_name = 'HS_' + hotspot + '_wavelengths.npy'
wavelengths_all = np.load(os.path.join(waveletOut_path, hotspot, wavelengths_file_name))

# Dataframe with info about the hotspots
hotspot_data = pd.read_excel(hotspotData_path)
    
# Get the original MBES data in Stream Coords 
# Open dictionary for reading Zmax
Zmax_name = 'HS_' + hotspot + '_max.json'
with open(os.path.join(StrCo_path, Zmax_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
Zmax_dict = {}
for key, value in serialized_dict.items():
    Zmax_dict[datetime.fromisoformat(key)] = np.array(value)    
    
# Open dictionary for reading Zmin    
Zmin_name = 'HS_' + hotspot + '_min.json'
with open(os.path.join(StrCo_path, Zmin_name), 'r') as f:
    # Load the JSON data from the file into a dictionary with string keys and nested list values
    serialized_dict = json.load(f)
# Convert string keys back to datetime objects, and nested lists to numpy arrays
Zmin_dict = {}
for key, value in serialized_dict.items():
    Zmin_dict[datetime.fromisoformat(key)] = np.array(value)        
 
# Load file info data about this hotspot that has inputs linking to the dictionary
file_info = pd.read_pickle(fileInfo_path)
file_info = file_info[file_info['Hotspot Letter'] == hotspot]
file_info = file_info[file_info.isin(Zmax_dict.keys()).any(axis=1)]
file_info.reset_index(inplace=True, drop=True)

# Create an array of Date Time objects for all time steps
datetime_array = file_info['Date Time'].values
 
# Load the average bed profile of the hotspot
Zavg_bed = np.load(os.path.join(waveletOut_path, hotspot, 'average_bed.npy')) 

# Load dredging and dumping dictionaries
name_dredge = 'HS_' + hotspot + '_dredging.pickle'
name_dump = 'HS_' + hotspot + '_dumping.pickle'
# Open a file for writing in binary mode
with open(os.path.join(hotspot_dredging_out_path, name_dredge), 'rb') as f:
    # Use the pickle.load() function to deserialize the dictionary from the file
    dredging_dict = pickle.load(f)
# Open a file for writing in binary mode
with open(os.path.join(hotspot_dredging_out_path, name_dump), 'rb') as f:
    # Use the pickle.load() function to deserialize the dictionary from the file
    dumping_dict = pickle.load(f)
    
# Determine the minimum and maximum x and y values in the hotspot from QGis
min_x, max_x = hotspot_data.loc[index, 'xmin'], hotspot_data.loc[index, 'xmax']
min_y, max_y = hotspot_data.loc[index, 'ymin'], hotspot_data.loc[index, 'ymax']

# # Loads a dataframe with line coordinates for the plough (developed by script Pepijn and Andries)
# lijnen = pd.read_pickle(os.path.join(ploeg_path,'lijnen.pkl'))
# # Create a GeoDataFrame of the plough_data
# lijnen = gpd.GeoDataFrame(lijnen)


shapefile_path = r'C:/Users/kruis/Documents/Thesis/Python/Lijnen/lijnen.shp'
data = gpd.read_file(shapefile_path)
lijnen= gpd.GeoDataFrame(data)
lijnen.set_index(['n_track', 'datum'], inplace=True)
lijnen = lijnen.cx[min_x:max_x, min_y:max_y]
lijnen = lijnen.reset_index()
# Transform plough lines to stream coords and turn them into polygons by adding the plough width
plough_strco = daf.plough_lines_to_polygons(lijnen, sn, plough_width, xs_min)

#%% Generate a 4D array containing 9 different bathymetries at each time interval

# Array with strings representing the different bathymetries
Znames = ['Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf']
# Determine all bathymetries for a chosen set of wavelengths
bathies0 = daf.generate_bathies(file_info.iloc[:], Znames, Zavg_bed, Zmax_dict, Zmin_dict, 
                           wavelengths_all, lowerbound, upperbound, 
                           waveletOut_path, hotspot)
bathies0 = bathies0[:,:,wxsmin:wxsmax,wdsmin:wdsmax]

# average displacement
# # Determine average displacement to next time step with Normalized Spatial Cross 
# Correlation on the Full Bathymetry
file_info = file_info.loc[file_info['TP Value']==0]
UP_indices = np.array(file_info.index)
file_info['Tindex*'] = UP_indices
bathies = bathies0[UP_indices,:,:,:]
file_info = file_info.reset_index(drop=True)

ds_shift_reg_array = np.full(len(file_info), np.nan)
for time in range(len(file_info)-1):
    time_diff = file_info.loc[time+1, 'Date Time'] - file_info.loc[time, 'Date Time']
    time_diff_weeks = time_diff.total_seconds() / (60*60*24*7)
    ds_shift_reg_array[time], ccf_reg = daf.nscc_full_bathymetry(bathies, time, time_diff_weeks)
    print(str(time)+'/'+str(len(file_info)))
file_info['ds_shift'] = ds_shift_reg_array
file_info = file_info.loc[(file_info['ds_shift'] > 2) | (file_info.index == len(file_info) -1)]

file_info_name = 'HS_'+hotspot+'_FI_test3.pkl'
file_info.to_pickle(os.path.join(out_path,file_info_name))
FIBackup = file_info.copy()

#%% Initialize time step for the dredged area that must be followed
file_info_name = 'HS_'+hotspot+'_FI_test3.pkl'
file_info = pd.read_pickle(os.path.join(out_path,file_info_name)) 
file_info = file_info.reset_index(drop=True)
                               ### Check time step
ds_shift_reg_array = np.array(file_info['ds_shift'])

# Select version of the bathymetry to be used
# pick from these: 'Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf'
Zname = 'Zrec'
# Set window sizes for NSCC.
ds_factor = 2          # 2 = window length equal to dune length in one dimensional direction
window_size_xs = 50     # 50 = window lenght of 100 m, covers a large part of the river width
# Set treshold parameter for dunes
threshold_height = -0.3

for time in range(len(file_info)):

    print(time)
    # Rework settings
    Zname_index = Znames.index(Zname)
    time_diff = file_info.loc[time+1, 'Date Time'] - file_info.loc[time, 'Date Time']
    time_diff_weeks = time_diff.total_seconds() / (60*60*24*7)
    
    # Determine average displacement to next time step with Normalized Spatial Cross 
    # Correlation on the Full Bathymetry
    # ds_shift_reg, ccf_reg = daf.nscc_full_bathymetry(bathies, time, time_diff_weeks)
    ds_shift_reg = ds_shift_reg_array[time]

    # Determine full crest locations
    crests = daf.find_local_maximums(bathies, time, sn, order=10, threshold_height=0)

    # Determine plough intensity
    if time==0:
        plough_intensity = np.full((Zavg_bed.shape[0], Zavg_bed.shape[1]), np.nan)
    else:
        plough_data = plough_strco.loc[plough_strco['date'].dt.date > file_info.loc[time-1, 'Date Time'].date()].loc[plough_strco['date'].dt.date <= file_info.loc[time, 'Date Time'].date()]
        plough_data.reset_index(drop=True)
        plough_data['id'] = range(1, len(plough_data)+1) # assign a unique ID to each polygon
        plough_data = plough_data.loc[:, ['poly', 'id']]
        plough_intensity = daf.rasterize_polygons(plough_data, Zavg_bed)
        plough_intensity = np.where(plough_intensity == 0, np.nan, plough_intensity)
    
    margin = 3
    for i, row in crests.iterrows():
        ds, xs = int(row['ds']), int(row['xs'])
        if not np.any(~np.isnan(plough_intensity[xs-margin:xs+margin,ds-margin:ds+margin])):
            crests.drop(i, inplace=True)
    
    if not len(crests) > 0:
        continue
    
    # Track dredged dunes
    # Determine analysis points
    tracker = daf.determine_analysis_points(crests, xs_min=wxsmin, xs_max=wxsmax, ds_min=wdsmin, 
                                      ds_max=wdsmax, minimum_points=3, interval=3)
    
    fig, ax = plt.subplots(1,1, figsize=(10,5))
    for Tdredg in range(9):
    
        # Determine dredging quantities on a grid
        dredging_grid = daf.calculate_quantity_grid(dredging_dict, file_info, time+Tdredg, Zavg_bed)
        
        # Determine dumping quantities on a grid
        dumping_grid = daf.calculate_quantity_grid(dumping_dict, file_info, time+Tdredg, Zavg_bed)
        
        # Determine plough intensity
        if time+Tdredg==0:
            plough_intensity = np.full((Zavg_bed.shape[0], Zavg_bed.shape[1]), np.nan)
        else:
            plough_data = plough_strco.loc[plough_strco['date'].dt.date > file_info.loc[time+Tdredg-1, 'Date Time'].date()].loc[plough_strco['date'].dt.date <= file_info.loc[time+Tdredg, 'Date Time'].date()]
            plough_data.reset_index(drop=True)
            plough_data['id'] = range(1, len(plough_data)+1) # assign a unique ID to each polygon
            plough_data = plough_data.loc[:, ['poly', 'id']]
            plough_intensity = daf.rasterize_polygons(plough_data, Zavg_bed)
            plough_intensity = np.where(plough_intensity == 0, np.nan, plough_intensity)
            
        # Determine dune length in river axis direction
        dunes = daf.get_dune_length_ds(bathies, time+Tdredg, tracker, upperbound, lowerbound, Zname='Zrec')
        
        # Perform NSCC on the analysis points
        dunes = daf.nssc_individual_analysis_points(bathies, dunes, time+Tdredg, ds_shift_reg_array[time+Tdredg], 
                                                    ds_factor, window_size_xs, Zname_index)
        
        # Transform coordinates to regular XY values again
        dunes = daf.convertToXY(dunes, sn, 'ds_new*', 'xs_new*' , 'X_new', 'Y_new')
    
        # Determine migration parameters per analysis point
        dunes['migration SC'] = np.sqrt(dunes['xs_shift']**2 + dunes['ds_shift']**2)          
        dunes['migration XY'] = np.sqrt((dunes['X_new']-(dunes['X']))**2 + (dunes['Y_new']-(dunes['Y']))**2) 
        dunes['migration angle'] = np.degrees(np.arctan(dunes['xs_shift']/dunes['ds_shift']))
        dunes['celerity SC'] = dunes['migration SC'] / time_diff_weeks
        dunes['celerity XY'] = dunes['migration XY'] / time_diff_weeks
           
        print('average ccf = ' + str(dunes['ccf'].mean()))
    
        # Calculate the dune parameters
        dunes = daf.calculate_dune_parameters(dunes, bathies, dredging_grid, dumping_grid, 
                                              plough_intensity, file_info, time+Tdredg, upperbound, 
                                              lowerbound, threshold_height)
        dunes = dunes.reset_index(drop=True)
        
        if Tdredg == 0:
            dunes['ID'] = dunes.index
            # dunes['H_0'] = dunes['height']
            dunes_all = dunes
            
        else:
            # colname = 'H_'+str(Tdredg)
            # dunes_all = dunes_all.merge(dunes[['ID', 'height']], on='ID', how='left').rename(columns={'height': colname})
            dunes_all = pd.concat([dunes_all, dunes], axis=0)
            
        dunes = dunes.dropna()
        tracker = dunes[['ds', 'xs', 'ds*', 'xs*', 'number', 'ID']].copy() 
        tracker['ds'] = tracker['ds']+dunes['ds_shift']
        tracker['xs'] = tracker['xs']+dunes['xs_shift']
        tracker['ds*'] = tracker['ds*']+dunes['ds_shift']
        tracker['ds*'] = tracker['ds*']+dunes['ds_shift']
        tracker = daf.convertToXY(tracker, sn, 'ds*', 'xs*' , 'X', 'Y')
    
        ### plot  
        if Tdredg == 0:
            im = ax.imshow(np.flip(bathies[time+Tdredg,Znames.index('Zrec'),wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='lower', vmin=-2, vmax=2) # set vmin and vmax to the minimum and maximum values in the array
            ax.scatter(dunes['ds']-wdsmin, dunes['xs*'], s=3, c='black')
        ax.scatter(dunes['ds']+dunes['ds_shift']-wdsmin, dunes['xs*']+dunes['xs_shift'], s=3, c='green')
        ax.plot([dunes['ds']-wdsmin,dunes['ds']+dunes['ds_shift']-wdsmin],[dunes['xs*'],dunes['xs*']+dunes['xs_shift']], linewidth=0.3, c='green')
        
        if len(tracker) == 0: break
    
    dunes_all = dunes_all.sort_values(by=['ID', 'time index'])
    dunes_all = dunes_all.reset_index(drop=True)
    
    dunes_all_name = str(time) + '.pkl'
    dunes_all.to_pickle(os.path.join(out_path,hotspot,'Ploeg_new',dunes_all_name))


#%%









#%%

# # Assuming your DataFrame is named 'df'
# rows_to_plot = dunes_all.loc[dunes_all['height_x'] > 1].iloc[:69]

# # Select the columns to plot (45 to 52)
# columns_to_plot = dunes_all.columns[35:43]

# fig = plt.subplots(figsize=(15,7))
# # Plot each row as a separate line
# for i, row in rows_to_plot.iterrows():
#     plt.plot(file_info['Date Time'].iloc[time:time+Tdredg], np.diff(row[columns_to_plot]), linewidth = 0.5, color='black' )

# plt.plot(file_info['Date Time'].iloc[time:time+Tdredg], np.diff(rows_to_plot[columns_to_plot].mean()), color='red')

# # Set labels and title
# plt.xlabel('Columns')
# plt.ylabel('Growth rate      mpt not in m / day yet')
# plt.title('Lines for Rows 1-20')

# # Display the plot
# plt.show()
























#%% Plot Dredging
fig, ax = plt.subplots(figsize=(15,7))

im0 = ax.imshow(np.flip(bathies[time,4,wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation)
im1 = ax.imshow(np.flip(plough_intensity[wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='Oranges', origin='upper')
im2 = ax.imshow(np.flip(dredging_grid[wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='spring', origin='upper')
im3 = ax.imshow(np.flip(dumping_grid[wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='winter', origin='upper')
# Create the colorbar and adjust its position
cbar0 = fig.colorbar(im0, ax=ax, orientation='horizontal', aspect=50, pad=0.05)
cbar0.ax.set_xlabel('Elevation [m]')
cbar1 = fig.colorbar(im1, ax=ax, orientation='horizontal', aspect=50, pad=0.10)
cbar1.ax.set_xlabel('Number of plough movements [-]')
cbar2 = fig.colorbar(im2, ax=ax, orientation='horizontal', aspect=50, pad=0.15)
cbar2.ax.set_xlabel('Dredged volume (=depth on 1x1 grid) [m3]')
cbar3 = fig.colorbar(im3, ax=ax, orientation='horizontal', aspect=50, pad=0.20)
cbar3.ax.set_xlabel('Dumped volume (=height on 1x1 grid) [m3]')
ax.set_xlabel('ds* [m]')
ax.set_ylabel('xs* [m]')
scatter = crests.loc[((crests['ds'] >= wdsmin) & (crests['ds'] <= wdsmax)) & ((crests['xs'] >= wxsmin) & (crests['xs'] <= wxsmax))]
ax.scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')      #!!! Check -1
plt.title('Dredging locations HS-F, time: ' + str(str(file_info.loc[time, 'Date Time'])), fontsize=16, fontweight='bold')
plt.tight_layout()
plt.savefig(os.path.join(out_path,'test','pres','dredging_locations.png'), dpi = 300)







#%% Plot Paper: NSCC



dunes = dunes.loc[((dunes['ds'] >= wdsmin) & (dunes['ds'] <= wdsmax)) & ((dunes['xs'] >= wxsmin) & (dunes['xs'] <= wxsmax))]
dunes = dunes.loc[(dunes['ds']+dunes['ds_shift'] <= wdsmax) & (dunes['xs*']+dunes['xs_shift'] < wxsmax+xs_min) & (dunes['xs*']+dunes['xs_shift'] > wxsmin+xs_min)]
scatter = crests.loc[((crests['ds'] >= wdsmin) & (crests['ds'] <= wdsmax)) & ((crests['xs'] >= wxsmin) & (crests['xs'] <= wxsmax))]


# Select version of the bathymetry to be used
plot_map = 'Zrec'     # pick from these: ['Z', 'Zmax', 'Zmin', 'Zavg', 'Zrec', 'Zf', 'Zmaxf', 'Zminf', 'Zavgf']

fig, ax = plt.subplots(2,1, figsize=(10,5))
im = ax[0].imshow(np.flip(bathies[time,Znames.index(plot_map),wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation) # set vmin and vmax to the minimum and maximum values in the array


# ax.scatter(troughs['ds'], troughs['xs'], s=0.01, c='black')
# ax[0].scatter(crests['ds']-wdsmin, crests['xs*'], s=0.01, alpha=0.5, c='black')
ax[0].scatter(dunes['ds']-wdsmin, dunes['xs*']*-1, s=3, c='black')
ax[0].scatter(dunes['ds']+dunes['ds_shift']-wdsmin, (dunes['xs*']+dunes['xs_shift'])*-1, s=3, c='green')
ax[0].plot([dunes['ds']-wdsmin,dunes['ds']+dunes['ds_shift']-wdsmin],[dunes['xs*']*-1,(dunes['xs*']+dunes['xs_shift'])*-1], linewidth=0.3, c='green')

ax[0].set_xlabel('ds* [m]')
ax[0].set_ylabel('xs* [m]')

time0 = file_info.loc[time, 'Date Time']
ax[0].set_title(time0)

im = ax[1].imshow(np.flip(bathies[time+1,Znames.index(plot_map),wxsmin:wxsmax,wdsmin:wdsmax], axis=1), extent=extent, cmap='bwr', origin='upper', vmin=-elevation, vmax=elevation) # set vmin and vmax to the minimum and maximum values in the array

# Create the colorbar and adjust its position

divider0 = make_axes_locatable(ax[0])
cax0 = divider0.append_axes('top', size='10%', pad=0.5)
cax0.set_xticks([])
cax0.set_yticks([])
cax0.set_frame_on(False)

divider = make_axes_locatable(ax[1])
cax = divider.append_axes('bottom', size='10%', pad=0.5)
cbar = fig.colorbar(im, cax=cax, orientation='horizontal')
cbar.set_label('Z [m]')
cbar.ax.xaxis.labelpad = 10

ax[1].scatter(dunes['ds']-wdsmin, dunes['xs*']*-1, s=3, c='black')
ax[1].scatter(dunes['ds']+dunes['ds_shift']-wdsmin, (dunes['xs*']+dunes['xs_shift'])*-1, s=3, c='green')
ax[1].plot([dunes['ds']-wdsmin,dunes['ds']+dunes['ds_shift']-wdsmin],[dunes['xs*']*-1,(dunes['xs*']+dunes['xs_shift'])*-1], linewidth=0.3, c='green')

ax[1].set_xlabel('ds* [m]')
ax[1].set_ylabel('xs* [m]')

time1 = file_info.loc[time+1, 'Date Time']
ax[1].set_title(time1)

ax[0].scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')
ax[1].scatter(scatter['ds']-wdsmin, scatter['xs*']*-1, s=0.01, alpha=1, c='black')

ax[0].text(.01, .98, 'a', ha='left', va='top', transform=ax[0].transAxes)
ax[1].text(.01, .98, 'b', ha='left', va='top', transform=ax[1].transAxes)

td = time1 - time0
fig.suptitle('HS-F: Spatial Cross Correlation - Time Interval: ' + str(td), fontsize=14)
print('Correlation map: ' + Zname)
print('Plot map: ' + plot_map)
plt.tight_layout()
# plt.savefig(os.path.join(paper_path,'nscc.png'), dpi = 300)












#%% Follow dredged area for next time steps



# # Determine search area (keep in mind, only xs is multiplied by 2)
# # if ds_shift_reg < 20:
# #     # if mean shift is very small, set some default values for the search area
# #     range_ds_min, range_ds_max = 0, 40
# #     range_xs = 5
# # else:
# #     # else set ds search area between 50% and 150% of the mean migration
# #     range_ds_min, range_ds_max = round(ds_shift_reg * 0.5), round(ds_shift_reg * 1.5)
# #     range_xs = round(ds_shift_reg/4)


# xs_shift = np.zeros(len(dunes))
# ds_shift = np.zeros(len(dunes))
# ccf = np.zeros(len(dunes))


# ds, xs = round(dredging_grid.shape[1]/2), round(dredging_grid.shape[0]/2)
# window_size_ds = ds
# window_size_xs = xs

# # Get window
# window0 = daf.getWindow(bathies[time, 4, :, :], xs, ds, window_size_xs, window_size_ds)
# window0 = window0[:-1, 1:-1]
# # Replace NaN values in window with dredging_grid values from array2
# nan_indices = np.isnan(dredging_grid)
# window0[nan_indices] = dredging_grid[nan_indices]

# time_ev = 5
# xs_shift = np.zeros(time_ev)
# ds_shift = np.zeros(time_ev)
# ccf = np.zeros(time_ev)

# for timestep in range(time_ev):
#     print(timestep)

#     if ds_shift_reg_array[time+timestep] < 20:
#         range_ds_min = round(0 + ds_shift_reg_array[time:time+timestep].sum())
#         range_ds_max = round(2*ds_shift_reg_array[time+timestep] + ds_shift_reg_array[time:time+timestep].sum())
#         range_xs = 10
#     else:
#         range_ds_min = round(ds_shift_reg_array[time+timestep] * 0.5 + ds_shift_reg_array[time:time+timestep].sum())
#         range_ds_max = round(ds_shift_reg_array[time+timestep] * 1.5 + ds_shift_reg_array[time:time+timestep].sum())
#         range_xs = 10
            
#     print(range_ds_min)
#     print(range_ds_max)   
#     ccc = np.zeros((range_xs * 2 + 1, range_ds_max - range_ds_min))
    
#     for dds in range(range_ds_max - range_ds_min):
#         print(str(dds) + '/' + str(range_ds_max - range_ds_min))
#         ds1 = ds + range_ds_min + dds
#         for dxs in range(range_xs * 2 + 1):
#             xs1 = xs - range_xs + dxs
    
#             window1 = daf.getWindow(bathies[time + timestep+1, Zname_index, :, :], xs1, ds1, window_size_xs, window_size_ds)
#             window1 = window1[:-1, 1:-1]
#             window1[nan_indices] = dredging_grid[nan_indices]
    
#             top = (window0 - np.nanmean(window0)) * (window1 - np.nanmean(window1))
#             bot1 = (window0 - np.nanmean(window0)) ** 2
#             bot2 = (window1 - np.nanmean(window1)) ** 2
#             ccc[dxs, dds] = np.nansum(top) / (np.nansum(bot1) * np.nansum(bot2)) ** 0.5
    
#     ccc = np.nan_to_num(ccc, nan=0)
#     xs_shift[timestep], ds_shift[timestep] = np.unravel_index(np.argmax(ccc), ccc.shape)
    
#     xs_shift[timestep] -= range_xs
#     ds_shift[timestep] += range_ds_min
#     ccf[timestep] = ccc.max()





















